package com.gic.enterprise.service;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.PushMessageDTO;
import com.gic.enterprise.dto.PushTypeDTO;
import com.gic.enterprise.dto.UserMessageDTO;

import java.util.Date;

public interface PushMessageApiService {

    /** @Description: 保存通知类型
     * @author taogs
     * @Date 16:52 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Integer> savePushType(PushTypeDTO pushTypeDTO);


    /** @Description: 分页查询通知类型
     * @author taogs
     * @Date 16:52 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Page<PushTypeDTO>> pagePushType(String search, int pageNum, int pageSize);


    /** @Description: 通过id获取通知类型
     * @author taogs
     * @Date 16:53 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<PushTypeDTO> getPushTypeById(Integer pushTypeId);

    /** @Description: 删除通知类型
     * @author taogs
     * @Date 16:53 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Integer> deletePushType(Integer pushTypeId);

    /** @Description: 保存通知消息
     * @author taogs
     * @Date 16:53 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Integer> savePushMessage(PushMessageDTO pushMessageDTO, int saveType);

    /** @Description: 分页查询通知消息
     * @author taogs
     * @Date 16:53 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Page<PushMessageDTO>> pagePushMessage(String search, Integer pushStatus, Date startTime,
                                                          Date endTime, Integer pageNum, Integer pageSize, Integer pushClassifyId);


    /** @Description: 获取通知详情
     * @author taogs
     * @Date 16:54 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<PushMessageDTO> getPushMessageById(Integer messageId);

    /** @Description: 删除通知消息
     * @author taogs
     * @Date 16:54 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Integer> deletePushMessage(Integer messageId);

    /** @Description: 定时器消息发送
     * @author taogs
     * @Date 16:54 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Void> pushMessageTask(String param);

    /** @Description: 队列任务消费
     * @author taogs
     * @Date 16:55 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Void> consumeMessageTask(String param);

    /** @Description: 查询我的消息
     * @author taogs
     * @Date 16:55 2019/11/27
     * @Param
     * @return
    */
    ServiceResponse<Page<UserMessageDTO>> pageUserMessage(Integer userId, String search, Integer status,
                                                          Date startTime, Date endTime, Integer pushClassifyId,
                                                          Integer pageNum, Integer pageSize);

    
    /** @Description: 撤销通知
     * @author taogs
     * @Date 17:00 2019/11/27
     * @Param 
     * @return 
    */
    ServiceResponse<Void> recallMessage(Integer messageId);

    
    /** @Description: 消息总数
     * @author taogs
     * @Date 17:00 2019/11/27
     * @Param 
     * @return 
    */
    ServiceResponse<Integer> totalUserMessage(Integer userId, Integer status);

    /** @Description: 推送消息接口
     * @author taogs
     * @Date 17:00 2019/11/27
     * @Param 
     * @return 
    */
    ServiceResponse<Integer> pushUserMessage(Integer enterpriseId, Integer userId, String classify, String title, String content);


    
    /** @Description: 商户下所有管理员推送消息
     * @author taogs
     * @Date 15:23 2020/3/9
     * @Param 
     * @return 
    */
    ServiceResponse<Integer> pushMessage(Integer enterpriseId, String classify, String title, String content);

    /** @Description: 所有标记为已读
     * @author taogs
     * @Date 17:01 2019/11/27
     * @Param 
     * @return 
    */
    ServiceResponse<Integer> readAll(Integer userId);

    /** @Description: 设置消息为已读
     * @author taogs
     * @Date 19:00 2019/11/27
     * @Param 
     * @return 
    */
    ServiceResponse<Integer> readMessage(Integer messageId, Integer userId);
}
