package com.gic.enterprise.service.impl;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dao.mapper.TabBillingEnterpriseSmsPackageMapper;
import com.gic.enterprise.dto.BillingEnterpriseSmsPackageDTO;
import com.gic.enterprise.entity.TabBillingEnterpriseSmsPackage;
import com.gic.enterprise.service.BillingEnterpriseSmsPackageService;
import com.gic.redis.data.util.RedisUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RAtomicLong;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service("billingEnterpriseSmsPackageService")
public class BillingEnterpriseSmsPackageServiceImpl implements BillingEnterpriseSmsPackageService {

    private Logger logger = LogManager.getLogger(BillingEnterpriseSmsPackageServiceImpl.class);
    @Autowired
    private TabBillingEnterpriseSmsPackageMapper tabBillingEnterpriseSmsPackageMapper;

    @Override
    public List<BillingEnterpriseSmsPackageDTO> listSmsPackage(Integer enterpriseId) {
        List<TabBillingEnterpriseSmsPackage> list = tabBillingEnterpriseSmsPackageMapper.listSmsPackage(enterpriseId);
        if (CollectionUtils.isNotEmpty(list)) {
            return EntityUtil.changeEntityListNew(BillingEnterpriseSmsPackageDTO.class, list);
        }
        return new ArrayList<>();
    }

    @Override
    public TabBillingEnterpriseSmsPackage getSmsPackageDetail(Integer id) {
        return this.tabBillingEnterpriseSmsPackageMapper.selectByPrimaryKey(id);
    }

    @Override
    public void saveEnterpriseSmsPackage(TabBillingEnterpriseSmsPackage smsPackage) {
        this.tabBillingEnterpriseSmsPackageMapper.insert(smsPackage);
    }

    @Override
    public void update(BillingEnterpriseSmsPackageDTO dto) {
        dto.setUpdateTime(new Date());
        tabBillingEnterpriseSmsPackageMapper
                .updateByPrimaryKeySelective(EntityUtil.changeEntityNew(TabBillingEnterpriseSmsPackage.class, dto));
    }

    @Override
    public Integer deduct(Integer enterpriseId) {
        TabBillingEnterpriseSmsPackage billingEnterpriseSmsPackage = tabBillingEnterpriseSmsPackageMapper.getEarliest(enterpriseId);
        Integer line = tabBillingEnterpriseSmsPackageMapper.deduct(billingEnterpriseSmsPackage.getEnterpriseSmsPackageId());
        if (line == 0) {
            // 重新确认下是否是没有余量了
            TabBillingEnterpriseSmsPackage smsPackage = tabBillingEnterpriseSmsPackageMapper.selectByPrimaryKey(billingEnterpriseSmsPackage.getEnterpriseSmsPackageId());
            if (smsPackage.getRemainAccount() == 0) {
                smsPackage.setStatus(3);
                tabBillingEnterpriseSmsPackageMapper.updateByPrimaryKeySelective(smsPackage);
            }
            deduct(enterpriseId);
        }
        return billingEnterpriseSmsPackage.getEnterpriseSmsPackageId();
    }

    @Override
    public RAtomicLong getTotalSmsInPackageByEnterpriseId(Integer enterpriseId) {
        RedissonClient redisClient = RedisUtil.getRedisClient();
        RAtomicLong smsCountR = redisClient.getAtomicLong(Constants.ENTERPRISE_BILLING_SMS_PACKAGE_COUNT + enterpriseId);
        long smsPackageCount = smsCountR.get();
        if (smsPackageCount == 0) {
            RedisUtil.lock(Constants.ENTERPRISE_BILLING_SMS_PACKAGE_COUNT_LOCK + enterpriseId, 30L);
            logger.info("redis中没有套餐包 查询数据库:{}", enterpriseId);
            Integer totalSms = tabBillingEnterpriseSmsPackageMapper.getTotalSmsInPackageByEnterpriseId(enterpriseId);
            smsCountR.set(totalSms);
            RedisUtil.unlock(Constants.ENTERPRISE_BILLING_SMS_PACKAGE_COUNT_LOCK + enterpriseId);
        }
        return smsCountR;
    }
}
