package com.gic.enterprise.service.impl;

import java.util.Date;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.commons.util.GlobalInfo;
import com.gic.enterprise.constant.BillingAuditStatusEnum;
import com.gic.enterprise.constant.PayTypeEnum;
import com.gic.enterprise.dao.mapper.TabBillingRechargeRecordMapper;
import com.gic.enterprise.dto.BillingRechargeDTO;
import com.gic.enterprise.entity.TabBillingRechargeRecord;
import com.gic.enterprise.qo.RechargeRecordQO;
import com.gic.enterprise.service.BillingAccountService;
import com.gic.enterprise.service.BillingRechargeRecordService;
import com.gic.enterprise.utils.CreateRandomUtils;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/15 17:05
 */
@Service
public class BillingRechargeRecordServiceImpl implements BillingRechargeRecordService {

    @Autowired
    private TabBillingRechargeRecordMapper tabBillingRechargeRecordMapper;
    @Autowired
    private BillingAccountService billingAccountService;

    @Override
    public TabBillingRechargeRecord saveOffLinePay(BillingRechargeDTO billingRechargeDTO) {
        TabBillingRechargeRecord rechargeRecord = new TabBillingRechargeRecord();
        rechargeRecord.setEnterpriseId(billingRechargeDTO.getEnterpriseId());
        rechargeRecord.setSerialNumber(CreateRandomUtils.createSerialNumber());
        rechargeRecord.setTotalFee(billingRechargeDTO.getTotalFee());
        rechargeRecord.setPayType(PayTypeEnum.OFFLINE_PAY.getCode());
        rechargeRecord.setPayInfoId(billingRechargeDTO.getPayInfoId());
        rechargeRecord.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        rechargeRecord.setInitiator(billingRechargeDTO.getInitiator());
        rechargeRecord.setInitiatorUser(billingRechargeDTO.getInitiatorUser());
        rechargeRecord.setInitiatorName(billingRechargeDTO.getInitiatorName());
        rechargeRecord.setAuditStatus(BillingAuditStatusEnum.AUDITING.getCode());
        rechargeRecord.setVerificationCode(billingRechargeDTO.getVerificationCode());
        rechargeRecord.setCreateTime(new Date());
        tabBillingRechargeRecordMapper.insertSelective(rechargeRecord);
        return rechargeRecord;
    }

    @Override
    public Integer saveWxPay(BillingRechargeDTO billingRechargeDTO) {
        TabBillingRechargeRecord rechargeRecord = new TabBillingRechargeRecord();
        rechargeRecord.setEnterpriseId(billingRechargeDTO.getEnterpriseId());
        rechargeRecord.setSerialNumber(billingRechargeDTO.getSerialNumber());
        rechargeRecord.setTotalFee(billingRechargeDTO.getTotalFee());
        rechargeRecord.setTotalFeePaid(billingRechargeDTO.getTotalFeePaid());
        rechargeRecord.setPayType(PayTypeEnum.WE_CHAT_PAY.getCode());
        rechargeRecord.setPayInfoId(billingRechargeDTO.getPayInfoId());
        rechargeRecord.setStatus(GlobalInfo.DATA_STATUS_NORMAL);
        rechargeRecord.setInitiator(billingRechargeDTO.getInitiator());
        rechargeRecord.setInitiatorUser(billingRechargeDTO.getInitiatorUser());
        rechargeRecord.setInitiatorName(billingRechargeDTO.getInitiatorName());
        rechargeRecord.setAuditStatus(BillingAuditStatusEnum.WAIT_PAY.getCode());
        rechargeRecord.setVerificationCode(billingRechargeDTO.getVerificationCode());
        rechargeRecord.setCreateTime(new Date());
        tabBillingRechargeRecordMapper.insertSelective(rechargeRecord);
        return rechargeRecord.getRechargeRecordId();
    }

    @Override
    public Page<TabBillingRechargeRecord> listRechargeRecord(RechargeRecordQO rechargeRecordQO) {
        PageHelper.startPage(rechargeRecordQO.getCurrentPage(), rechargeRecordQO.getPageSize());
        return tabBillingRechargeRecordMapper.listRechargeRecord(rechargeRecordQO);
    }

    @Override
    public void updateBySerialNumber(Integer enterpriseId, String orderSerialNumber, Double totalFee, Double totalFeePaid, Integer auditStatus, String timeEnd) {
        // 修改订单状态
        tabBillingRechargeRecordMapper.updateBySerialNumber(orderSerialNumber, totalFeePaid, auditStatus, timeEnd);
        // 给账户充值
        if (BillingAuditStatusEnum.PASS.getCode().equals(auditStatus)) {
            billingAccountService.updateAccountAfterRechargeOrRefund(totalFee, enterpriseId);
        }
    }

    @Override
    public TabBillingRechargeRecord getRechargeRecord(Integer rechargeRecordId) {
        return tabBillingRechargeRecordMapper.selectByPrimaryKey(rechargeRecordId);
    }

    @Override
    public void updateAudit(Integer auditStatus, Double accountAmount, Integer rechargeRecordId) {
        TabBillingRechargeRecord billingRechargeRecord = new TabBillingRechargeRecord();
        billingRechargeRecord.setAuditStatus(auditStatus);
        billingRechargeRecord.setTotalFeePaid(accountAmount);
        billingRechargeRecord.setRechargeRecordId(rechargeRecordId);
        tabBillingRechargeRecordMapper.updateByPrimaryKeySelective(billingRechargeRecord);
    }

    @Override
    public TabBillingRechargeRecord getAuditingRechargeRecord(Integer enterpriseId) {
        return tabBillingRechargeRecordMapper.getAuditingRechargeRecord(enterpriseId);
    }

    @Override
    public Integer cancelRechargeApproval(Integer rechargeRecordId, Integer auditStatus) {
        TabBillingRechargeRecord billingRechargeRecord = new TabBillingRechargeRecord();
        billingRechargeRecord.setRechargeRecordId(rechargeRecordId);
        billingRechargeRecord.setAuditStatus(auditStatus);
        return tabBillingRechargeRecordMapper.updateByPrimaryKeySelective(billingRechargeRecord);
    }

    @Override
    public TabBillingRechargeRecord getRechargeRecordBySerialNumber(String serialNumber) {
        return tabBillingRechargeRecordMapper.getRechargeRecordBySerialNumber(serialNumber);
    }

}
