package com.gic.enterprise.service.outer.impl;

import java.util.*;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.service.CustomStoreApiService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.constant.applet.AppletPageComponentEnum;
import com.gic.enterprise.constant.applet.AppletPageTypeEnum;
import com.gic.enterprise.constant.applet.EntryConditionEnum;
import com.gic.enterprise.dto.AppletCustomPageDTO;
import com.gic.enterprise.entity.TabAppletCustomPage;
import com.gic.enterprise.entity.TabAppletPageComponent;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.AppletPageQO;
import com.gic.enterprise.service.AppletCustomPageApiService;
import com.gic.enterprise.service.AppletCustomPageService;
import com.gic.enterprise.service.AppletPageComponentService;
import org.springframework.transaction.interceptor.TransactionAspectSupport;

@Service("appletCustomPageApiService")
public class AppletCustomPageApiServiceImpl implements AppletCustomPageApiService {
    private static final Logger LOGGER = LogManager.getLogger(AppletCustomPageApiServiceImpl.class);
    @Autowired
    private AppletCustomPageService appletCustomPageService;
    @Autowired
    private AppletPageComponentService appletPageComponentService;
    @Autowired
    private CustomStoreApiService customStoreApiService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public ServiceResponse<Integer> saveOrUpdate(String param, Integer pageId, Integer enterpriseId, Integer saveType,
            Integer appType, String appId) {
        //参数校验
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        //1：保存并发布 2：保存
        boolean isRightStatus = saveType == null || (saveType.intValue() != 1 && saveType.intValue() != 2);
        if (isRightStatus) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面状态值错误");
        }
        //1:小程序 2：服务号
        boolean isRightAppType = appType == null || (appType.intValue() != 1 && appType.intValue() != 2);
        if (isRightAppType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "app类型值错误");
        }
        if (StringUtils.isBlank(appId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "appKey/appId不能为空");
        }
        JSONObject jsonObject = JSONObject.parseObject(param);
        if (!jsonObject.containsKey("title")) {
            LOGGER.warn("title不存在");
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面名称错误");
        }
        String title = jsonObject.getString("title");
        if (StringUtils.isBlank(title)) {
            LOGGER.warn("title不存在");
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面名称错误");
        }
        if (!jsonObject.containsKey("entryCondition")) {
            LOGGER.warn("entryCondition不存在");
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面进入条件错误");
        }
        //0代表不限，1代表开卡会员，2代表认证会员
        Integer entryCondition = jsonObject.getInteger("entryCondition");
        if (!EntryConditionEnum.isRightCode(entryCondition)) {
            LOGGER.warn("entryCondition值错误");
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面进入条件错误");
        }
        if (!jsonObject.containsKey("pageType")) {
            LOGGER.warn("pageType不存在");
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面类型错误");
        }
        Integer pageType = jsonObject.getInteger("pageType");
        if (!AppletPageTypeEnum.isRightCode(pageType)) {
            LOGGER.warn("pageType值错误");
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面类型错误");
        }

        //填充实体类
        TabAppletCustomPage record = new TabAppletCustomPage();
        record.setEnterpriseId(enterpriseId);
        record.setStatus(saveType);
        record.setAppType(appType);
        record.setAppId(appId);
        //页面名称
        record.setTitle(title);
        // 0代表不限，1代表开卡会员，2代表认证会员
        record.setEntryCondition(entryCondition);
        record.setSetting(jsonObject.getString("setting"));
        record.setPageType(pageType);

        //如果是有默认数据的类型（会员中心），第一条记录，即使saveType = 2，也自动变成1
        if (AppletPageTypeEnum.hasDefault(pageType)) {
            //查询记录
            int count = appletCustomPageService.count(enterpriseId, appType, appId, pageType);
            boolean isNeedSet = false;

            if (pageId == null && count < 1) {
                isNeedSet = true;
            } else if (pageId != null && count == 1) {
                isNeedSet = true;
            }

            if (isNeedSet) {
                record.setStatus(1);
            }
        }

        //判断，如果是会员中心/客服页面，只能发布一条记录，自定义页面可以多条
        if (saveType.intValue() == 1) {
            //保存并发布
            if (AppletPageTypeEnum.isOnlyOneCode(pageType)) {
                //把其他的数据状态改为关闭
                appletCustomPageService.disableFetch(enterpriseId, appType, appId, pageType);
            }
        }

        //新增或者编辑
        if (pageId == null) {
            pageId = appletCustomPageService.save(record);
        } else {
            TabAppletCustomPage getById = appletCustomPageService.getByPageId(pageId);
            if (getById == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面ID错误，查无数据");
            }

            //
            if (getById.getEnterpriseId().intValue() == -1) {
                //如果编辑的是默认数据，则变成新增一条一摸一样的数据
                record.setStatus(1);
                pageId = appletCustomPageService.save(record);
            } else {
                record.setEnterpriseId(getById.getEnterpriseId());
                record.setPageId(pageId);
                if (record.getStatus().intValue() != 1) {
                    record.setStatus(getById.getStatus());
                }
                appletCustomPageService.edit(record);
            }
        }

        //组件解析
        JSONArray component = jsonObject.getJSONArray("value");
        //保存组件表数据
        List<TabAppletPageComponent> componentList = new ArrayList<>();
        Date now = new Date();
        //组件key存储
        Map<String, Object> componentMap = new HashMap<>(16);
        if (component != null) {
            for (int i = 0, length = component.size(); i < length; i++) {
                JSONObject json = component.getJSONObject(i);
                String code = json.getString("key");
                if(code.equals(AppletPageComponentEnum.MALL_GOODS.getCode())){
                    int goodsCount = json.getIntValue("goodsCount");
                    int size = json.getJSONArray("goods").size();
                    if(size > goodsCount){
                        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商品数量超出限制！");
                    }
                }
                Integer componentIndex = json.getInteger("elementIndex");
                if (AppletPageComponentEnum.isSaveToComponentTable(code)) {
                    //组件key存储
                    componentMap.put(code, componentIndex);

                    TabAppletPageComponent temp = new TabAppletPageComponent();
                    temp.setEnterpriseId(enterpriseId);
                    temp.setComponentIndex(componentIndex);
                    temp.setComponentType(code);
                    temp.setCreateTime(now);
                    temp.setUpdateTime(now);
                    temp.setSetting(json.toJSONString());
                    temp.setPageId(pageId);
                    temp.setStatus(1);
                    componentList.add(temp);
                }
            }
        }

        //如果是会员中心等页面，必须有对应的组件
        AppletPageComponentEnum mustComponent = AppletPageComponentEnum.mustComponent(pageType);
        if (mustComponent != null) {
            //说明该页面有必须的组件，需要验证组件是否存在
            if (!componentMap.containsKey(mustComponent.getCode())) {
                //回滚
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "必须有"
                                + mustComponent.getMessage() + "组件");
            }
        }

        //如果是会员中心，商户初始化回调
        if (pageType == AppletPageTypeEnum.MEMBER_CENTER.getCode()) {
            this.customStoreApiService.customSettingHasCommpleted(enterpriseId);
        }
        if (CollectionUtils.isNotEmpty(componentList)) {
            //先删除
            appletPageComponentService.deleteByPageId(pageId);
            appletPageComponentService.saveFetch(componentList);
        }
        return ServiceResponse.success(pageId);
    }

    @Override
    public ServiceResponse<Page<AppletCustomPageDTO>> page(AppletPageQO params) {
        com.github.pagehelper.Page page = appletCustomPageService.page(params);
        Page<AppletCustomPageDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page,
                AppletCustomPageDTO.class);
        List<AppletCustomPageDTO> list = resultPage.getResult();
        //如果查不到数据，使用默认的数据
        boolean isDefault = CollectionUtils.isEmpty(list) && params.getCurrentPage().intValue() == 1
                && params.getPageType() != null
                && AppletPageTypeEnum.MEMBER_CENTER.getCode() == params.getPageType().intValue();
        if (isDefault) {
            TabAppletCustomPage record = appletCustomPageService.getDefault(AppletPageTypeEnum.MEMBER_CENTER.getCode());
            list = new ArrayList<>();
            list.add(EntityUtil.changeEntityNew(AppletCustomPageDTO.class, record));
            resultPage.setResult(list);
            resultPage.setTotalCount(1);
            resultPage.setTotalPage(1);
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<AppletCustomPageDTO> getDetailByPageId(Integer pageId) {
        TabAppletCustomPage record = appletCustomPageService.getByPageId(pageId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面ID错误，无数据");
        }
        AppletCustomPageDTO dto = EntityUtil.changeEntityNew(AppletCustomPageDTO.class, record);
        //查询组件数据
        List<TabAppletPageComponent> componentList = appletPageComponentService.listByPageId(pageId);
        JSONArray component = new JSONArray();
        if (CollectionUtils.isNotEmpty(componentList)) {
            for (TabAppletPageComponent temp : componentList) {
                JSONObject jsonObject = JSONObject.parseObject(temp.getSetting());
                component.add(jsonObject);
            }
        }
        dto.setComponentStr(component.toJSONString());
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<AppletCustomPageDTO> getDetailByAppId(Integer enterpriseId, Integer appType, String appId,
            Integer pageType) {
        if (appType == null) {
            //默认查询小程序类型
            appType = 1;
        }
        if (StringUtils.isBlank(appId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "appId不能为空");
        }
        boolean isNotRightAppType = appType.intValue() != 1 && appType.intValue() != 2;
        if (isNotRightAppType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "app类型值错误");
        }
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (!AppletPageTypeEnum.isOnlyOneCode(pageType)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面类型值有误，非只能有一条发布状态的页面类型值");
        }
        List<TabAppletCustomPage> record = appletCustomPageService.listEnablePage(enterpriseId, appType, appId,
                pageType);
        //接口返回对象
        AppletCustomPageDTO dto = null;
        Integer pageId;
        if (CollectionUtils.isEmpty(record) && AppletPageTypeEnum.hasDefault(pageType)) {
            //如果为空，则查询默认数据
            TabAppletCustomPage defaultData = appletCustomPageService
                    .getDefault(pageType);
            dto = EntityUtil.changeEntityNew(AppletCustomPageDTO.class, defaultData);
            //查询组件数据
            pageId = defaultData.getPageId();
            dto.setComponentStr(getComponentByPageId(pageId));
        } else if (CollectionUtils.isNotEmpty(record)) {
            dto = EntityUtil.changeEntityNew(AppletCustomPageDTO.class, record.get(0));
            //查询组件数据
            dto.setComponentStr(getComponentByPageId(record.get(0).getPageId()));
        }
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<String> enable(Integer pageId) {
        TabAppletCustomPage record = appletCustomPageService.getByPageId(pageId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面ID错误，无数据");
        }
        if (record.getStatus().intValue() == 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已是发布状态");
        }
        if (AppletPageTypeEnum.isOnlyOneCode(record.getPageType())) {
            //把其他的数据状态改为关闭
            appletCustomPageService.disableFetch(record.getEnterpriseId(), record.getAppType(), record.getAppId(),
                    record.getPageType());
        }
        appletCustomPageService.updateStatus(pageId, 1);
        return ServiceResponse.success(record.getTitle());
    }

    @Override
    public ServiceResponse<String> disable(Integer pageId) {
        TabAppletCustomPage record = appletCustomPageService.getByPageId(pageId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面ID错误，无数据");
        }
        if (record.getStatus().intValue() != 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已是关闭状态");
        }
        if (AppletPageTypeEnum.isOnlyOneCode(record.getPageType())) {
            //如果只有一条，则不能关闭
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不能关闭，必须有一条是发布状态");
        }
        appletCustomPageService.updateStatus(pageId, 2);
        return ServiceResponse.success(record.getTitle());
    }

    @Override
    public ServiceResponse<String> delete(Integer pageId) {
        TabAppletCustomPage record = appletCustomPageService.getByPageId(pageId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面ID错误，无数据");
        }
        if (record.getStatus().intValue() == 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发布状态不能删除");
        }
        appletCustomPageService.updateStatus(pageId, 0);
        return ServiceResponse.success(record.getTitle());
    }

    @Override
    public ServiceResponse<String> deleteFetch(String pageIds) {
        if (StringUtils.isBlank(pageIds)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面id为空");
        }
        String[] pageIdArr = pageIds.split(",");
        StringBuilder pageName = new StringBuilder();
        List<Integer> pageIdList = new ArrayList<>(pageIdArr.length);
        for (String pageId : pageIdArr) {
            if (!StringUtils.isNumeric(pageId)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面id错误，数据格式非法");
            }
            TabAppletCustomPage record = appletCustomPageService.getByPageId(Integer.parseInt(pageId));
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "页面ID错误，查无数据");
            }
            if (record.getStatus().intValue() == 1) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "发布状态不能删除");
            }
            pageName.append(record.getTitle()).append(",");
            pageIdList.add(Integer.parseInt(pageId));
        }
        appletCustomPageService.deleteFetch(pageIdList);
        return ServiceResponse.success(pageName.toString());
    }

    /**
     * 获取组件信息
     * @param pageId
     * @return
     */
    private String getComponentByPageId(Integer pageId) {
        //查询组件数据
        List<TabAppletPageComponent> componentList = appletPageComponentService.listByPageId(pageId);
        JSONArray component = new JSONArray();
        if (CollectionUtils.isNotEmpty(componentList)) {
            for (TabAppletPageComponent temp : componentList) {
                JSONObject jsonObject = JSONObject.parseObject(temp.getSetting());
                component.add(jsonObject);
            }
        }
        return component.toJSONString();
    }

}
