package com.gic.enterprise.service.outer.impl;

import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import com.alibaba.fastjson.JSON;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.dto.AppletCardDTO;
import com.gic.enterprise.dto.AppletMainCardDTO;
import com.gic.enterprise.entity.TabAppletDecorationConfig;
import com.gic.member.config.api.dto.MemberCardRelDTO;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.enterprise.dto.AppletDataAssetAreaDTO;
import com.gic.enterprise.dto.AppletDecorationConfigDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.AppletDecorationConfigApiService;
import com.gic.enterprise.service.AppletDecorationConfigService;
import com.gic.mall.share.api.dto.shop.ShopDTO;
import com.gic.mall.share.api.service.ShopApiService;
import com.gic.member.config.api.dto.AppletCardRelationDTO;
import com.gic.member.config.api.dto.AppletsConfigSimpleOutputDTO;
import com.gic.member.config.api.service.AppletsConfigApiService;

@Service("appletDecorationConfigApiService")
public class AppletDecorationConfigApiServiceImpl implements AppletDecorationConfigApiService {
    private static final Logger LOGGER = LogManager.getLogger(AppletDecorationConfigApiServiceImpl.class);
    @Autowired
    private AppletDecorationConfigService appletDecorationConfigService;
    @Autowired
    private ShopApiService shopApiService;
    @Autowired
    private AppletsConfigApiService appletsConfigApiService;

    @Override
    public ServiceResponse<Integer> save(AppletDecorationConfigDTO dto) {

        ServiceResponse valueValidResult = valueValid(dto);
        if (!valueValidResult.isSuccess()) {
            return ServiceResponse.failure(valueValidResult.getCode(), valueValidResult.getMessage());
        }

        //
        TabAppletDecorationConfig record = appletDecorationConfigService.getByAppId(dto.getAppId(),
                dto.getEnterpriseId());
        if (record != null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "不得重复配置");
        }

        return ServiceResponse.success(appletDecorationConfigService.save(dto));
    }

    @Override
    public ServiceResponse<String> edit(AppletDecorationConfigDTO dto) {
        TabAppletDecorationConfig record = appletDecorationConfigService.getById(dto.getConfigId());
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "装修配置ID错误，查无数据");
        }
        dto.setAppId(record.getAppId());
        dto.setEnterpriseId(record.getEnterpriseId());

        ServiceResponse valueValidResult = valueValid(dto);
        if (!valueValidResult.isSuccess()) {
            return ServiceResponse.failure(valueValidResult.getCode(), valueValidResult.getMessage());
        }

        appletDecorationConfigService.edit(dto);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<AppletDecorationConfigDTO> getDetailByAppId(String appId, Integer enterpriseId) {
        TabAppletDecorationConfig record = appletDecorationConfigService.getByAppId(appId, enterpriseId);
        if (record == null) {
            //默认配置信息
            ServiceResponse<Long> mainCardResult = getMainCard(appId, enterpriseId);
            if (!mainCardResult.isSuccess()) {
                return ServiceResponse.failure(mainCardResult.getCode(), mainCardResult.getMessage());
            }
            //默认选中和主卡关联的店铺，不可编辑
            ServiceResponse<ShopDTO> shopResult = getMainCardStore(appId, enterpriseId);
            if (!shopResult.isSuccess()) {
                return ServiceResponse.failure(shopResult.getCode(), shopResult.getMessage());
            }
            ShopDTO shopDTO = shopResult.getResult();

            Long mainCardId = mainCardResult.getResult();
            AppletDecorationConfigDTO result = new AppletDecorationConfigDTO();
            //数据资产配置信息
            AppletDataAssetAreaDTO dataAssetArea = new AppletDataAssetAreaDTO();
            dataAssetArea.setOrder(mainCardId.toString());
            dataAssetArea.setCoupon(mainCardId.toString());
            dataAssetArea.setIntegral(mainCardId.toString());
            dataAssetArea.setMallOrder(shopDTO.getId().toString());
            dataAssetArea.setMallOrderName(shopDTO.getName());
            //会员中心页面卡范围
            result.setMemberCardArea(mainCardId.toString());
            //数据资产对象
            result.setAppletDataAssetArea(dataAssetArea);
            //数据资产json字符串
            result.setDataAssetArea(JSON.toJSONString(dataAssetArea));
            //GPS智能判定卡 默认关闭
            result.setGpsDecisionCard(0);
            result.setEnterpriseId(enterpriseId);
            result.setAppId(appId);
            //开卡认证关联,全部的卡列表
            //校验参数数值是否非法
            ServiceResponse<List<AppletCardDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
            if (!appletCardResult.isSuccess()) {
                return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
            }
            List<AppletCardDTO> appletCardList = appletCardResult.getResult();
            result.setOpenCardAuth(appletCardList.stream().map(e -> e.getCardConfigId().toString()).collect(Collectors.joining(",")));
            return ServiceResponse.success(result);
        }
        AppletDecorationConfigDTO dto = EntityUtil.changeEntityNew(AppletDecorationConfigDTO.class, record);
        AppletDataAssetAreaDTO old = JSON.parseObject(record.getDataAssetArea(), AppletDataAssetAreaDTO.class);
        dto.setAppletDataAssetArea(old);
        return ServiceResponse.success(dto);
    }

    @Override
    public ServiceResponse<Object> listAppletCard(String appId, Integer enterpriseId) {
        ServiceResponse<List<AppletCardDTO>> result = getMemberCardByAppId(appId, enterpriseId);
        if (result.isSuccess()) {
            return ServiceResponse.success(result.getResult());
        }
        return ServiceResponse.failure(result.getCode(), result.getMessage());
    }

    @Override
    public ServiceResponse<Object> listAppletCardOfTotalIntegral(String appId, Integer enterpriseId) {
        ServiceResponse<List<AppletCardDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
        if (!appletCardResult.isSuccess()) {
            return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
        }
        List<AppletMainCardDTO> result = new ArrayList<>();
        //默认主卡
        ServiceResponse<AppletCardRelationDTO> mainCardResult = getMainCardObj(appId, enterpriseId);
        if (!mainCardResult.isSuccess()) {
            return ServiceResponse.failure(mainCardResult.getCode(), mainCardResult.getMessage());
        }
        //添加主卡
        AppletCardRelationDTO mainCard = mainCardResult.getResult();

        String mainCardId = mainCard.getCardConfigId().toString();
        List<AppletCardDTO> appletCardList = appletCardResult.getResult();
        for (AppletCardDTO cardDTO : appletCardList) {
            List<AppletCardDTO> subList = cardDTO.getSubList();
            if (cardDTO.getCardConfigId().toString().equals(mainCardId)) {
                //为了获取名称
                result.add(new AppletMainCardDTO().setCardConfigId(mainCard.getCardConfigId())
                        .setCardName(cardDTO.getCardName()).setMainCard(true));
            }
            if (CollectionUtils.isNotEmpty(subList)) {
                for (AppletCardDTO cardDTO1 : subList) {
                    if (cardDTO1.getCardConfigId().toString().equals(mainCardId)) {
                        result.add(new AppletMainCardDTO().setCardConfigId(cardDTO.getCardConfigId())
                                .setCardName(cardDTO.getCardName()).setMainCard(false));
                    }
                }
            }
        }
        return ServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<Object> listAppletCardByCardId(String cardId, String appId, Integer enterpriseId) {
        //校验参数数值是否非法
        ServiceResponse<List<AppletCardDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
        if (!appletCardResult.isSuccess()) {
            return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
        }
        List<AppletCardDTO> appletCardList = appletCardResult.getResult();
        List<AppletCardDTO> result = new ArrayList<>();
        for (AppletCardDTO cardDTO : appletCardList) {
            if (cardDTO.getCardConfigId().toString().equals(cardId)) {
                result.add(new AppletCardDTO().setCardConfigId(cardDTO.getCardConfigId())
                        .setCardName(cardDTO.getCardName()));
                List<AppletCardDTO> subList = cardDTO.getSubList();
                if (CollectionUtils.isNotEmpty(subList)) {
                    for (AppletCardDTO sub : subList) {
                        result.add(sub);
                    }
                }
            }
        }
        return ServiceResponse.success(result);
    }

    private ServiceResponse valueValid(AppletDecorationConfigDTO dto) {
        String appId = dto.getAppId();
        Integer enterpriseId = dto.getEnterpriseId();
        AppletDataAssetAreaDTO appletDataAssetArea = dto.getAppletDataAssetArea();
        if (StringUtils.isBlank(appId)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "小程序ID不能为空");
        }
        if (enterpriseId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (appletDataAssetArea == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产配置不能为空");
        }

        //默认主卡
        ServiceResponse<Long> mainCardResult = getMainCard(appId, enterpriseId);
        if (!mainCardResult.isSuccess()) {
            return ServiceResponse.failure(mainCardResult.getCode(), mainCardResult.getMessage());
        }
        Long mainCardId = mainCardResult.getResult();

        //校验参数数值是否非法
        ServiceResponse<List<AppletCardDTO>> appletCardResult = getMemberCardByAppId(appId, enterpriseId);
        if (!appletCardResult.isSuccess()) {
            return ServiceResponse.failure(appletCardResult.getCode(), appletCardResult.getMessage());
        }
        List<AppletCardDTO> appletCardList = appletCardResult.getResult();
        Map<String, List<AppletCardDTO>> appletCardMap = appletCardList
                .stream()
                .collect(Collectors.toMap(e -> e.getCardConfigId().toString(), e -> e.getSubList() ));
        //子卡
        List<String> appletSubCardList = appletCardList
                .stream()
                .filter(e -> CollectionUtils.isNotEmpty(e.getSubList()))
                .filter(e -> {
                    List<AppletCardDTO> subList = e.getSubList();
                    return subList.stream()
                            .anyMatch(son -> son.getCardConfigId().toString().equals(mainCardId.toString()));
                })
                .map(e -> e.getCardConfigId().toString())
                .collect(Collectors.toList());

        //需要校验的数值集合
        Map<String, String> needValidMap = new HashMap<>(16);
        needValidMap.put("会员中心页面卡范围", dto.getMemberCardArea());
        needValidMap.put("开卡认证关联", dto.getOpenCardAuth());
        //数据资产
        //总积分
        String totalIntegral = appletDataAssetArea.getIntegral();
        //线下订单
        String order = appletDataAssetArea.getOrder();
        needValidMap.put("数据资产-总积分", totalIntegral);
        needValidMap.put("数据资产-卡券包", appletDataAssetArea.getCoupon());
        needValidMap.put("数据资产-线下订单", order);

        ServiceResponse valueValid;
        for (Map.Entry<String, String> entry : needValidMap.entrySet()) {
            valueValid = validParamValue(entry.getKey(), entry.getValue(), appletCardMap);
            if (!valueValid.isSuccess()) {
                return ServiceResponse.failure(valueValid.getCode(), valueValid.getMessage());
            }
        }

        if (totalIntegral.split(",").length > 1) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-总积分只能选择一个");
        }
//        if (order.split(",").length > 1) {
//            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-线下订单只能选择一个");
//        }
        if (!mainCardId.toString().equals(totalIntegral) && !appletSubCardList.contains(totalIntegral)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-总积分只能选择小程序主卡或者主卡关联的平衡主卡");
        }
        //如果总积分选择主卡，则线下订单也是主卡，不可变更
        if (mainCardId.toString().equals(totalIntegral)) {
            appletDataAssetArea.setOrder(totalIntegral);
        } else {
            //总积分平衡组卡下面的子卡列表，线下订单选择
            List<AppletCardDTO> subListTemp = appletCardMap.get(totalIntegral);
            Set<String> subListSet = new HashSet<>();
            subListSet.add(totalIntegral);
            if (CollectionUtils.isNotEmpty(subListTemp)) {
                for (AppletCardDTO temp : subListTemp) {
                    subListSet.add(temp.getCardConfigId().toString());
                }
            }
            if (!totalIntegral.toString().equals(order)) {
                boolean validOrder = Stream.of(order.split(","))
                        .anyMatch(e -> !subListSet.contains(e));
                if (validOrder) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "数据资产-线下订单只能选择总积分小程序主卡关联的平衡组卡及其子卡");
                }
            }
        }

        //默认主卡
        //签到
        appletDataAssetArea.setSignIn(mainCardId.toString());
        //会员成就
        appletDataAssetArea.setMemberAchievement(mainCardId.toString());
        //会员任务
        appletDataAssetArea.setMemberTask(mainCardId.toString());
        //关联卡组,全部的卡
        dto.setOpenCardAuth(appletCardList.stream().map(e -> e.getCardConfigId().toString()).collect(Collectors.joining(",")));

        Integer gpsDecisionCard = dto.getGpsDecisionCard();
        if (gpsDecisionCard == null) {
            //GPS智能判定卡 默认关闭
            dto.setGpsDecisionCard(0);
        } else {
            if (gpsDecisionCard != 1 && gpsDecisionCard != 0) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "GPS智能判定卡数值非法");
            }
        }

        //默认选中和主卡关联的店铺，不可编辑 //实时调用
        ServiceResponse<ShopDTO> shopResult = getMainCardStore(appId, enterpriseId);
        if (!shopResult.isSuccess()) {
            return ServiceResponse.failure(shopResult.getCode(), shopResult.getMessage());
        }
        ShopDTO shopDTO = shopResult.getResult();
        appletDataAssetArea.setMallOrder(shopDTO.getId().toString());
        appletDataAssetArea.setMallOrderName(shopDTO.getName());
        //数据资产设置
        dto.setDataAssetArea(JSON.toJSONString(appletDataAssetArea));

        dto.setAppletDataAssetArea(appletDataAssetArea);
        return ServiceResponse.success();
    }

    private ServiceResponse validParamValue(String key, String value, Map<String, List<AppletCardDTO>> appletCardMap) {
        if (StringUtils.isBlank(value)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), key + "不能为空");
        }
        String[] arr = value.split(",");
        for (String str : arr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            if (!appletCardMap.containsKey(str)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), key + "数值非法");
            }
        }
        return ServiceResponse.success();
    }

    /**
     * 查询会员卡列表
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<List<AppletCardDTO>> getMemberCardByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();

        //默认主卡
        ServiceResponse<Long> mainCardResult = getMainCard(appId, enterpriseId);
        String mainCardId = "";
        if (mainCardResult.isSuccess()) {
            mainCardId = mainCardResult.getResult().toString();
        }

        ServiceResponse<List<MemberCardRelDTO>> appletCardResult = appletsConfigApiService
                .queryAppletIntegralBalanceCardRel(appletConfigId, enterpriseId);
        if (appletCardResult.isSuccess()) {
            List<MemberCardRelDTO> cardList = appletCardResult.getResult();
            List<AppletCardDTO> resultList = new ArrayList<>();
            if (CollectionUtils.isNotEmpty(cardList)) {
                for (MemberCardRelDTO card : cardList) {
                    List<MemberCardRelDTO> sub = card.getRelCards();
                    Boolean isMainCard = card.getId().toString().equals(mainCardId);

                    List<AppletCardDTO> subList = new ArrayList<>();
                    if (CollectionUtils.isNotEmpty(sub)) {
                        for (MemberCardRelDTO subTemp : sub) {
                            Boolean isMainCardTemp = subTemp.getId().toString().equals(mainCardId);
                            subList.add(new AppletCardDTO().setCardConfigId(subTemp.getId())
                                    .setCardName(subTemp.getName()).setMainCard(isMainCardTemp));
                        }
                    }
                    resultList.add(new AppletCardDTO().setCardName(card.getName()).setCardConfigId(card.getId())
                            .setSubList(subList).setMainCard(isMainCard));
                }
            }
            return ServiceResponse.success(resultList);
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), appletCardResult.getMessage());
    }

    /**
     * 获取主卡
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<Long> getMainCard(String appId, Integer enterpriseId) {
        ServiceResponse<AppletCardRelationDTO> mainCard = getMainCardObj(appId, enterpriseId);
        if (!mainCard.isSuccess()) {
            return ServiceResponse.failure(mainCard.getCode(), mainCard.getMessage());
        }
        return ServiceResponse.success(mainCard.getResult().getCardConfigId());
    }

    private ServiceResponse<AppletCardRelationDTO> getMainCardObj(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();
        List<Long> param = new ArrayList<>(1);
        param.add(appletConfigId);
        ServiceResponse<List<AppletCardRelationDTO>> appletCardListResult = appletsConfigApiService
                .queryAppletCardRelation(param, enterpriseId);
        if (appletCardListResult.isSuccess()) {
            List<AppletCardRelationDTO> list = appletCardListResult.getResult();
            if (CollectionUtils.isNotEmpty(list)) {
                for (AppletCardRelationDTO dto : list) {
                    if (dto.getMainCard()) {
                        return ServiceResponse.success(dto);
                    }
                }
            }
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "没有主卡");
        }
        return ServiceResponse.failure(appletCardListResult.getCode(), appletCardListResult.getMessage());
    }

    /**
     * 获取和主卡关联的店铺
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<ShopDTO> getMainCardStore(String appId, Integer enterpriseId) {
        ServiceResponse<Long> appletResult = getAppletConfigIdByAppId(appId, enterpriseId);
        if (!appletResult.isSuccess()) {
            return ServiceResponse.failure(appletResult.getCode(), appletResult.getMessage());
        }
        long appletConfigId = appletResult.getResult();
        //shopType 1:微商城 2：积分
        ServiceResponse<ShopDTO> shopResult = shopApiService.getShopByAppletId(appletConfigId, enterpriseId, 1);
        if (shopResult.isSuccess()) {
            return ServiceResponse.success(shopResult.getResult());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), shopResult.getMessage());
    }

    /**
     * 根据appId查询小程序配置ID
     * @param appId
     * @param enterpriseId
     * @return
     */
    private ServiceResponse<Long> getAppletConfigIdByAppId(String appId, Integer enterpriseId) {
        ServiceResponse<AppletsConfigSimpleOutputDTO> appletResult = appletsConfigApiService
                .selectAppletsConfigIdByAppId(appId, enterpriseId);
        if (appletResult.isSuccess()) {
            return ServiceResponse.success(appletResult.getResult().getAppletConfigId());
        }
        return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), appletResult.getMessage());
    }

}
