package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.constant.LoginUserTypeEnum;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.AuditorApiService;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.DateUtil;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GlobalInfo;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.config.Config;
import com.gic.enterprise.constant.AuditTypeEnum;
import com.gic.enterprise.constant.DataSecurityAuditEnum;
import com.gic.enterprise.constants.Constants;
import com.gic.enterprise.dto.AuditDownloadDTO;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.dto.AuditRiskModeDTO;
import com.gic.enterprise.dto.TodoItemDTO;
import com.gic.enterprise.entity.*;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.AuditLogQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.AsyncCallbackUtils;
import com.gic.marketing.process.api.service.tmpl.WxTemplateRouterApiService;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-24 16:36
 */
@Service("auditLogApiService")
public class AuditLogApiServiceImpl implements AuditLogApiService {

    private Logger logger = LogManager.getLogger(AuditLogApiServiceImpl.class);

    @Autowired
    private AuditLogService auditLogService;
    @Autowired
    private AuditProcessService auditProcessService;
    @Autowired
    private AuditorApiService auditorApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private ProjectItemService projectItemService;
    @Autowired
    private RiskModeService riskModeService;
    @Autowired
    private DownloadReportService downloadReportService;
    @Autowired
    private TodoItemApiService todoItemApiService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private PushMessageApiService pushMessageApiService;
    @Autowired
    private WxTemplateRouterApiService wxTemplateRouterApiService;
    @Autowired
    private Config config;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Void> apply(AuditLogDTO auditLogDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(auditLogDTO, AuditLogDTO.ApplyValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        if (auditLogDTO.getAuditType() == null) {
            auditLogDTO.setAuditType(AuditTypeEnum.NORMAL.getCode());
        }
        // 判断审批项是否存在
        TabProjectItem projectItem = projectItemService.getByProjectItemCode(auditLogDTO.getProjectItemCode());
        if (projectItem == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        auditLogDTO.setProjectItemId(projectItem.getProjectItemId());
        auditLogDTO.setProjectItemName(projectItem.getProjectItemName());
        // 判断申请人是否存在
        ServiceResponse<UserDTO> userServiceResponse = userApiService.getUserInfoById(auditLogDTO.getApplyUserId());
        if (!userServiceResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        auditLogDTO.setApplyUserName(userServiceResponse.getResult().getUserName());
        auditLogDTO.setApplyUserPhone(userServiceResponse.getResult().getPhoneNumber());
        // 判断有没有审核员， 没有审核员就返回错误
        ServiceResponse<List<AuditorDTO>> auditorResponse = auditorApiService.listAuditorByProject(auditLogDTO.getProjectItemId(), auditLogDTO.getApplyUserId(), auditLogDTO.getEnterpriseId());
        if (!auditorResponse.isSuccess()) {
            return EnterpriseServiceResponse.failure(auditorResponse.getCode(), auditorResponse.getMessage());
        }
        List<AuditorDTO> auditorList = auditorResponse.getResult();
        List<AuditorDTO> openidList = auditorList.stream().filter(e -> StringUtils.isNotBlank(e.getOpenid())).collect(Collectors.toList());
        List<AuditorDTO> userIdList = auditorList.stream().filter(e -> e.getUserId() != null).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(openidList) && CollectionUtils.isEmpty(userIdList)) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "审核员没有绑定微信或管理员");
        }

        // 判断审批项是否禁用
        if (Constants.DEL_STATUS.equals(projectItem.getUseStatus())) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "该审批项已被禁用");
        }

        // 如果存在5分钟外未审核的申请， 把这个申请设为过期
        TabAuditLog lastAudit = auditLogService.getLastWaitAuditByApplyUser(auditLogDTO.getEnterpriseId(), auditLogDTO.getApplyUserId(), auditLogDTO.getProjectItemId());
        if (lastAudit != null) {
            lastAudit.setAuditExpireTime(new Date());
            auditLogService.update(lastAudit);
        }

        // 保存日志
        if (auditLogDTO.getApplyTime() == null) {
            auditLogDTO.setApplyTime(new Date());
        }
        if (auditLogDTO.getAuditExpireTime() == null) {
            auditLogDTO.setAuditExpireTime(DateUtil.addDay(auditLogDTO.getApplyTime(), projectItem.getAuditExpireDuration()));
        }
        auditLogDTO.setAuditResult(DataSecurityAuditEnum.WAIT.getCode());
        Integer auditLogId = auditLogService.save(auditLogDTO);

        for (AuditorDTO auditorDTO : auditorList) {
            if (StringUtils.isNotBlank(auditorDTO.getOpenid())) {
                // 发送微信模板消息
                sendWxMessage(auditorDTO, auditLogDTO);
            }
            if (auditorDTO.getUserId() != null) {
                ServiceResponse<UserDTO> userResponse = userApiService.getUserInfoById(auditorDTO.getUserId());
                if (userResponse.isSuccess()) {
                    UserDTO userDTO = userResponse.getResult();
                    if (userDTO != null) {
                        TodoItemDTO todoItemDTO = new TodoItemDTO();
                        // 商户id
                        todoItemDTO.setEnterpriseId(userDTO.getEnterpriseId());
                        // 待办事项的userId
                        todoItemDTO.setUserId(userDTO.getUserId());
                        // 事项详情
                        todoItemDTO.setItemContent(projectItem.getProjectItemDesc());
                        // 事项code
                        todoItemDTO.setItemType(com.gic.enterprise.constant.Constants.TODO_ITEM_AUDIT);
                        // todo 事项url
                        todoItemDTO.setItemInfoUrl(com.gic.enterprise.constant.Constants.TODO_ITEM_AUDIT_URL + "?auditLogId=" + auditLogId);
                        todoItemApiService.saveTodoItem(todoItemDTO);
                    }
                }
            }
        }

        if (Constants.NORMAL_STATUS.equals(projectItem.getIsPlatform())) {
            // todo 如果是平台审批项， 还要发送给实施  一期先不做
            // sendWxMessage();
        }

        return EnterpriseServiceResponse.success();
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<AuditLogDTO> audit(AuditLogDTO auditLogDTO) {
        // 完成待办事项
        todoItemApiService.finishTodoItem(auditLogDTO.getEnterpriseId(), auditLogDTO.getAuditorId(), com.gic.enterprise.constant.Constants.TODO_ITEM_AUDIT, "www.baidu.com?auditLogId=" + auditLogDTO.getAuditLogId());

        TabAuditLog auditLog = this.auditLogService.getAuditById(auditLogDTO.getAuditLogId());
        if (auditLog == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        // 判断当前的审批是否是未审批的状态 已经通过的话就返回
        if (!DataSecurityAuditEnum.WAIT.getCode().equals(auditLog.getAuditResult())) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "该申请已经被审核通过", EntityUtil.changeEntityByJSON(AuditLogDTO.class, auditLog));
        }
        if (DataSecurityAuditEnum.WAIT.getCode().equals(auditLog.getAuditResult()) && DateUtil.compareDate(new Date(), auditLog.getAuditExpireTime())) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "申请已过期");
        }

        TabProjectItem projectItem = projectItemService.getById(auditLog.getProjectItemId());
        if (projectItem == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
        // 记录审批流程
        TabAuditProcess auditProcess = getProcess(auditLog, auditLogDTO, projectItem);
        this.auditProcessService.save(auditProcess);
        // todo 如果是平台级审批项 要先判断是否实施和管理员都通过了  一期先不做
//        if (Constants.NORMAL_STATUS.equals(projectItem.getIsPlatform())) {
//            TabAuditProcess otherTypeProcess = this.auditProcessService.getOtherTypeProcess(auditProcess.getEnterpriseId(), auditProcess.getAuditLogId(), auditProcess.getAuditorType());
//            if (otherTypeProcess == null) {
//                // 实施和管理员没有都审批通过 直接返回
//                return EnterpriseServiceResponse.success();
//            }
//        }
        // 修改审批状态
        auditLogService.updateAudit(auditLogDTO);
        // 回调
        String extraInfo = auditLog.getExtraInfo();
        JSONObject jsonObject = JSON.parseObject(extraInfo);
        jsonObject.put("auditResult", auditLogDTO.getAuditResult());
        jsonObject.put("auditorId", auditLogDTO.getAuditorId());
        jsonObject.put("auditorPhone", auditLogDTO.getAuditorPhone());
        jsonObject.put("auditorName", auditLogDTO.getAuditorName());
        jsonObject.put("auditReason", auditLogDTO.getAuditReason());
        String[] callback = projectItem.getCallbackUrl().split(GlobalInfo.FLAG_COMMA);
        AsyncCallbackUtils.callBack(jsonObject, callback[0], callback[1]);
        //发送通知消息
        String url = config.getHost() + "/damo-system/examine-logs";
        String result = auditLogDTO.getAuditResult() == 1? "已通过" : "未通过";
        String content = "您的操作"+ result +"审核，可以在<a href='"+url+"'>审核日志</a>查看到本次审核记录";
        this.pushMessageApiService.pushUserMessage(auditLogDTO.getEnterpriseId(), auditLogDTO.getApplyUserId(),"审核通知", "审核结果", content);
        return EnterpriseServiceResponse.success();
    }

    private TabAuditProcess getProcess(TabAuditLog auditLog, AuditLogDTO auditLogDTO, TabProjectItem projectItemDTO) {
        TabAuditProcess auditProcess = new TabAuditProcess();
        auditProcess.setEnterpriseId(auditLog.getEnterpriseId());
        auditProcess.setAuditLogId(auditLog.getAuditLogId());
        auditProcess.setProjectItemId(auditLog.getProjectItemId());
        auditProcess.setIsPlatform(projectItemDTO.getIsPlatform());
        auditProcess.setAuditorId(auditLogDTO.getAuditorId());
        auditProcess.setAuditResult(auditLogDTO.getAuditResult());
        // TODO 用户类型
//        auditProcess.setAuditorType(auditLogDTO.getLoginType());
        return auditProcess;
    }

    @Override
    public ServiceResponse<AuditLogDTO> getAuditLog(Integer auditLogId) {
        TabAuditLog audit = this.auditLogService.getAuditById(auditLogId);
        AuditLogDTO auditLogDTO;
        if (audit != null) {
            if (AuditTypeEnum.RISK_MODE.getCode().equals(audit.getAuditType())) {
                AuditRiskModeDTO auditRiskModeDTO = EntityUtil.changeEntityByJSON(AuditRiskModeDTO.class, audit);
                JSONObject jsonObject = JSON.parseObject(auditRiskModeDTO.getExtraInfo());
                TabRiskMode riskMode = riskModeService.getById(jsonObject.getInteger("riskModeId"));
                auditRiskModeDTO.setDuration(riskMode.getDurationCode());
                auditRiskModeDTO.setApplyReason(riskMode.getApplyReason());
                auditLogDTO = auditRiskModeDTO;
//                return EnterpriseServiceResponse.success(auditRiskModeDTO);
            } else if (AuditTypeEnum.DOWNLOAD_REPORT.getCode().equals(audit.getAuditType())) {
                AuditDownloadDTO downloadDTO = EntityUtil.changeEntityByJSON(AuditDownloadDTO.class, audit);
                JSONObject jsonObject = JSON.parseObject(downloadDTO.getExtraInfo());
                downloadDTO.setDownloadReportId(jsonObject.getInteger("downloadReportId"));
                TabDownloadReport downloadReport = downloadReportService.getById(downloadDTO.getDownloadReportId());
                downloadDTO.setDataContent(downloadReport.getDataContent());
                downloadDTO.setDataCount(downloadReport.getDataCount());
                downloadDTO.setReportStartTime(downloadReport.getReportStartTime());
                downloadDTO.setReportEndTime(downloadReport.getReportEndTime());
                auditLogDTO = downloadDTO;
//                return EnterpriseServiceResponse.success(downloadDTO);
            } else {
                auditLogDTO = EntityUtil.changeEntityByJSON(AuditLogDTO.class, audit);
//                return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(AuditLogDTO.class, audit));
            }
            // 商户名
            TabEnterprise enterprise = enterpriseService.getEnterpriseById(auditLogDTO.getEnterpriseId());
            auditLogDTO.setEnterpriseName(enterprise.getEnterpriseName());
            // 职位
            ServiceResponse<UserDTO> user = userApiService.getUserInfoById(auditLogDTO.getApplyUserId());
            if (user.isSuccess()) {
                UserDTO userDTO = user.getResult();
                if (LoginUserTypeEnum.OPERATION_USER.getCode() == userDTO.getLoginType()) {
                    auditLogDTO.setPosition(LoginUserTypeEnum.OPERATION_USER.getMessage());
                } else if (userDTO.getSuperAdmin() == 1) {
                    auditLogDTO.setPosition("超级管理员");
                } else {
                    auditLogDTO.setPosition("管理员");
                }
                auditLogDTO.setApplyUserPhone(userDTO.getPhoneNumber());
            }
            // 受审项目名
            TabProject project = projectService.getProjectByProjectItem(auditLogDTO.getProjectItemId());
            auditLogDTO.setProjectName(project.getProjectName());
            //
            return EnterpriseServiceResponse.success(auditLogDTO);
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Page<AuditLogDTO>> listAuditLog(AuditLogQO auditLogQO) {
        com.github.pagehelper.Page<TabAuditLog> auditLogList = this.auditLogService.listAuditLog(auditLogQO);
        Date currDate = new Date();
        if (CollectionUtils.isNotEmpty(auditLogList)) {
            for (TabAuditLog auditLog : auditLogList) {
                if (DateUtil.compareDate(currDate, auditLog.getAuditExpireTime())) {
                    auditLog.setAuditResult(DataSecurityAuditEnum.OVER_TIME.getCode());
                }
            }
        }
        Page<AuditLogDTO> page = PageHelperUtils.changePageHelperToCurrentPage(auditLogList, AuditLogDTO.class);
        return EnterpriseServiceResponse.success(page);
    }

    private void sendWxMessage(AuditorDTO auditorDTO, AuditLogDTO auditLogDTO) {
        TabEnterprise enterprise = enterpriseService.getEnterpriseById(auditLogDTO.getEnterpriseId());
        UserDTO userDTO = userApiService.getUserById(auditLogDTO.getApplyUserId()).getResult();
        LoginUserTypeEnum userTypeEnum = LoginUserTypeEnum.getByCode(userDTO.getLoginType());
        String position = "";
        if (userTypeEnum != null) {
            position = userTypeEnum.getMessage();
        }
        String[] keyWord = {enterprise.getEnterpriseName(), auditorDTO.getAuditorName(), position, auditLogDTO.getProjectItemName()};
        // todo 跳转地址
        this.wxTemplateRouterApiService.sendForDownloadData(auditorDTO.getOpenid(), null, keyWord);
    }

}
