package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.dto.rule.RuleEnterpriseDTO;
import com.gic.enterprise.entity.TabConfigRule;
import com.gic.enterprise.entity.TabConfigRuleEnterprise;
import com.gic.enterprise.entity.TabConfigRuleEnterpriseRel;
import com.gic.enterprise.entity.TabConfigRuleRel;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.ConfigRuleEnterpriseApiService;
import com.gic.enterprise.service.ConfigRuleEnterpriseService;
import com.gic.enterprise.service.ConfigRuleService;
import com.gic.enterprise.service.EnterpriseService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;
import java.util.stream.Collectors;

@Service("configRuleEnterpriseApiService")
public class ConfigRuleEnterpriseApiServiceImpl implements ConfigRuleEnterpriseApiService {
    @Autowired
    private ConfigRuleEnterpriseService configRuleEnterpriseService;
    @Autowired
    private EnterpriseService enterpriseService;
    @Autowired
    private ConfigRuleService configRuleService;

    @Override
    public ServiceResponse<Integer> save(RuleEnterpriseDTO dto) {
        Integer enterpriseRuleId = dto.getEnterpriseRuleId();

        if (enterpriseRuleId != null) {
            TabConfigRuleEnterprise record = configRuleEnterpriseService.getByEnterpriseRuleId(enterpriseRuleId);
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，无记录");
            }
        }

        ServiceResponse<Void> valid = valid(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        //验证并获取商户ID列表
        boolean isDefaultRule = "-1".equals(dto.getEnterpriseIds());
        List<Integer> enterpriseIdList;

        if (!isDefaultRule) {
            //如果不是平台默认规则，则需要进行商户的校验
            ServiceResponse<List<Integer>> enterpriseIdListResult = getEnterpriseIdList(dto);
            if (enterpriseIdListResult.isSuccess()) {
                enterpriseIdList = enterpriseIdListResult.getResult();
            } else {
                return ServiceResponse.failure(enterpriseIdListResult.getCode(), enterpriseIdListResult.getMessage());
            }
        } else {
            if (enterpriseRuleId == null) {
                //新增的情况
                TabConfigRuleEnterpriseRel defaultRule = configRuleEnterpriseService.getByEnterpriseId(-1);
                if (defaultRule != null) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "平台默认规则重复配置");
                }
            }
            //全部商户
            dto.setEnterpriseType(2);
            enterpriseIdList = Arrays.asList(-1);
        }

        //验证并获取规则树数据
        ServiceResponse<List<Integer>> ruleIdListResult = getRuleIdList(dto);
        List<Integer> ruleIdList;
        if (ruleIdListResult.isSuccess()) {
            ruleIdList = ruleIdListResult.getResult();
        } else {
            return ServiceResponse.failure(ruleIdListResult.getCode(), ruleIdListResult.getMessage());
        }

        if (enterpriseRuleId == null) {
            //新增商户规则
            enterpriseRuleId = configRuleEnterpriseService.save(dto);
        } else {
            configRuleEnterpriseService.update(new TabConfigRuleEnterprise().setEnterpriseRuleId(enterpriseRuleId)
                    .setRuleName(dto.getRuleName()).setEnterpriseType(dto.getEnterpriseType()));
        }

        //新增商户和规则关联表数据
        List<TabConfigRuleEnterpriseRel> ruleEnterpriseRelList = new ArrayList<>(enterpriseIdList.size());
        Date now = new Date();
        for (Integer enterpriseId : enterpriseIdList) {
            ruleEnterpriseRelList.add(new TabConfigRuleEnterpriseRel().setCreateTime(now).setUpdateTime(now)
                    .setEnterpriseId(enterpriseId).setStatus(1).setEnterpriseRuleId(enterpriseRuleId));
        }
        //解绑
        configRuleEnterpriseService.deleteByEnterpriseRuleId(enterpriseRuleId);
        configRuleEnterpriseService.deleteByEnterpriseIdList(enterpriseIdList);
        //绑定
        configRuleEnterpriseService.insertBatch(ruleEnterpriseRelList);

        //新增规则树和商户规则关联数据
        List<TabConfigRuleRel> ruleRelList = new ArrayList<>(ruleIdList.size());
        for (Integer ruleId : ruleIdList) {
            ruleRelList.add(new TabConfigRuleRel().setCreateTime(now).setUpdateTime(now).setStatus(1).setRuleId(ruleId)
                    .setEnterpriseRuleId(enterpriseRuleId));
        }
        //解绑
        configRuleEnterpriseService.deleteByEnterpriseRuleIdForRuleRel(enterpriseRuleId);
        //绑定
        configRuleEnterpriseService.insertBatchForRuleRel(ruleRelList);
        return ServiceResponse.success(enterpriseRuleId);
    }

    @Override
    public ServiceResponse<Integer> edit(RuleEnterpriseDTO dto) {
        return save(dto);
    }

    @Override
    public ServiceResponse<List<String>> hasRuleOfToBeSetEnterprises(RuleEnterpriseDTO dto) {
        Integer enterpriseRuleId = dto.getEnterpriseRuleId();

        if (enterpriseRuleId != null) {
            TabConfigRuleEnterprise record = configRuleEnterpriseService.getByEnterpriseRuleId(enterpriseRuleId);
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，无记录");
            }
        }

        ServiceResponse<Void> valid = valid(dto);
        if (!valid.isSuccess()) {
            return ServiceResponse.failure(valid.getCode(), valid.getMessage());
        }
        //验证并获取商户ID列表
        boolean isDefaultRule = "-1".equals(dto.getEnterpriseIds());
        List<String> enterpriseIdList;

        if (!isDefaultRule) {
            //如果不是平台默认规则，则需要进行商户的校验
            ServiceResponse<List<Integer>> enterpriseIdListResult = getEnterpriseIdList(dto);
            if (enterpriseIdListResult.isSuccess()) {
                List<Integer> tempEnterpriseIdList = enterpriseIdListResult.getResult();
                enterpriseIdList = Optional.ofNullable(tempEnterpriseIdList).orElse(new ArrayList<>())
                        .stream()
                        .map(e -> e.toString())
                        .collect(Collectors.toList());
            } else {
                return ServiceResponse.failure(enterpriseIdListResult.getCode(), enterpriseIdListResult.getMessage());
            }
        } else {
            return ServiceResponse.success();
        }

        List<EnterpriseDTO> enterpriseDTOList = enterpriseService.listEnterprise();
        Map<String, String> enterpriseIdMapName = Optional.ofNullable(enterpriseDTOList).orElse(new ArrayList<>())
                .stream()
                .collect(Collectors.toMap(e -> e.getEnterpriseId().toString(), EnterpriseDTO::getEnterpriseName));
        //查询规则关联商户数据
        List<TabConfigRuleEnterpriseRel> rel = configRuleEnterpriseService.listEnterprise();
        return ServiceResponse.success(Optional.ofNullable(rel).orElse(new ArrayList<>())
                .stream()
                .filter(e -> Optional.ofNullable(enterpriseRuleId).orElse(-111).intValue() != e.getEnterpriseId().intValue()
                    && enterpriseIdList.contains(e.getEnterpriseId().toString()))
                .map(e -> enterpriseIdMapName.get(e.getEnterpriseId().toString()))
                .collect(Collectors.toList()));
    }

    @Override
    public ServiceResponse<String> deleteEnterpriseRule(Integer enterpriseRuleId) {
        TabConfigRuleEnterprise record = configRuleEnterpriseService.getByEnterpriseRuleId(enterpriseRuleId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，无记录");
        }
        TabConfigRuleEnterpriseRel defaultRule = configRuleEnterpriseService.getByEnterpriseId(-1);
        if (defaultRule != null && defaultRule.getEnterpriseRuleId().intValue() == enterpriseRuleId) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "平台默认规则不允许删除");
        }
        configRuleEnterpriseService.update(record.setStatus(0));
        //解绑
        configRuleEnterpriseService.deleteByEnterpriseRuleId(enterpriseRuleId);
        configRuleEnterpriseService.deleteByEnterpriseRuleIdForRuleRel(enterpriseRuleId);
        return ServiceResponse.success(record.getRuleName());
    }

    @Override
    public ServiceResponse<String> removeEnterprise(Integer enterpriseId) {
        if (enterpriseId == -1) {
            //防止把平台规则商户记录删除
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则ID错误，非法操作");
        }
        List<Integer> enterpriseIdList = new ArrayList<>(1);
        enterpriseIdList.add(enterpriseId);
        configRuleEnterpriseService.deleteByEnterpriseIdList(enterpriseIdList);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Page<RuleEnterpriseDTO>> listRule(String search, Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = configRuleEnterpriseService.listRule(search, pageNum, pageSize);
        Page<RuleEnterpriseDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        //
        if (pageNum != null && pageNum == 1) {
            //添加默认平台规则
            TabConfigRuleEnterpriseRel defaultRule = configRuleEnterpriseService.getByEnterpriseId(-1);
            if (defaultRule != null) {
                List<RuleEnterpriseDTO> resultList = resultPage.getResult();
                TabConfigRuleEnterprise record = configRuleEnterpriseService
                        .getByEnterpriseRuleId(defaultRule.getEnterpriseRuleId());
                if (CollectionUtils.isEmpty(resultList)) {
                    resultList = new ArrayList<>(1);
                }
                //过滤默认规则，防止重复
                resultList = resultList
                        .stream()
                        .filter(e -> !e.getEnterpriseRuleId().toString().equals(defaultRule.getEnterpriseRuleId().toString()))
                        .collect(Collectors.toList());
                resultList.add(0, new RuleEnterpriseDTO().setEnterpriseRuleId(defaultRule.getEnterpriseRuleId())
                        .setRuleName(record.getRuleName()).setEnterpriseId(-1));
                resultPage.setResult(resultList);
            }
        }
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Page<RuleEnterpriseDTO>> listEnterpriseRule(String search, Integer enterpriseRuleId,
            Integer pageNum, Integer pageSize) {
        com.github.pagehelper.Page page = configRuleEnterpriseService.listEnterpriseRule(search, enterpriseRuleId,
                pageNum, pageSize);
        Page<RuleEnterpriseDTO> resultPage = PageHelperUtils.changePageHelperToCurrentPage(page);
        return ServiceResponse.success(resultPage);
    }

    @Override
    public ServiceResponse<Boolean> hasDefaultEnterpriseRule() {
        TabConfigRuleEnterpriseRel defaultRule = configRuleEnterpriseService.getByEnterpriseId(-1);
        if (defaultRule != null) {
            return ServiceResponse.success(true);
        }
        return ServiceResponse.success(false);
    }

    /**
     * 验证参数
     * @param dto
     * @return
     */
    private ServiceResponse<Void> valid(RuleEnterpriseDTO dto) {
        if (StringUtils.isBlank(dto.getRuleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称不能为空");
        }
        Integer enterpriseType = dto.getEnterpriseType();
        boolean isNotRightEnterpriseType = enterpriseType == null
                || (enterpriseType.intValue() != 1 && enterpriseType.intValue() != 2);
        if (isNotRightEnterpriseType) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户类型值非法");
        }
        if (enterpriseType.intValue() == 1) {
            //部分商户
            if (StringUtils.isBlank(dto.getEnterpriseIds())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "部分商户数据为空");
            }
        }
        if (StringUtils.isBlank(dto.getRuleIds())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则树数据为空");
        }
        if (configRuleEnterpriseService.isRepeatName(dto.getEnterpriseRuleId(), dto.getRuleName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则名称重复");
        }
        return ServiceResponse.success();
    }

    private ServiceResponse<List<Integer>> getEnterpriseIdList(RuleEnterpriseDTO dto) {
        Integer enterpriseType = dto.getEnterpriseType();
        List<Integer> enterpriseIdList = new ArrayList<>();
        //已存在的商户ID
        Set<String> enterpriseSet = new HashSet<>();
        List<EnterpriseDTO> enterpriseDTOList = enterpriseService.listEnterprise();
        if (CollectionUtils.isNotEmpty(enterpriseDTOList)) {
            for (EnterpriseDTO enterpriseDTO : enterpriseDTOList) {
                enterpriseSet.add(enterpriseDTO.getEnterpriseId().toString());
                enterpriseIdList.add(enterpriseDTO.getEnterpriseId());
            }
        }
        //用于过滤重复ID
        Set<String> enterpriseTempSet = new HashSet<>();
        if (enterpriseType.intValue() == 1) {
            String[] arr = dto.getEnterpriseIds().split(",");
            enterpriseIdList.clear();
            for (String str : arr) {
                if (StringUtils.isBlank(str)) {
                    continue;
                }
                if (!StringUtils.isNumeric(str)) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "部分商户数据值非法");
                }
                Integer id = Integer.parseInt(str);
                if (!enterpriseSet.contains(id.toString())) {
                    return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "部分商户数据错误，不存在的商户");
                }
                enterpriseTempSet.add(str);
            }
            //循环装载
            for (String str : enterpriseTempSet) {
                enterpriseIdList.add(Integer.parseInt(str));
            }
        }
        return ServiceResponse.success(enterpriseIdList);
    }

    private ServiceResponse<List<Integer>> getRuleIdList(RuleEnterpriseDTO dto) {
        List<Integer> ruleIdList = new ArrayList<>();
        Set<String> ruleIdSet = new HashSet<>();
        List<TabConfigRule> ruleList = configRuleService.listAll(null);
        if (CollectionUtils.isNotEmpty(ruleList)) {
            for (TabConfigRule rule : ruleList) {
                ruleIdSet.add(rule.getRuleId().toString());
            }
        }
        //用于过滤重复ID
        Set<String> ruleIdTempSet = new HashSet<>();
        String[] arr = dto.getRuleIds().split(",");
        for (String str : arr) {
            if (StringUtils.isBlank(str)) {
                continue;
            }
            if (!StringUtils.isNumeric(str)) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则树数据值非法");
            }
            Integer id = Integer.parseInt(str);
            if (!ruleIdSet.contains(id.toString())) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "规则树数据错误，不存在的规则");
            }
            ruleIdTempSet.add(str);
        }
        //循环装载
        for (String str : ruleIdTempSet) {
            ruleIdList.add(Integer.parseInt(str));
        }
        return ServiceResponse.success(ruleIdList);
    }

}
