package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.service.AuditorApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.ProjectItemDTO;
import com.gic.enterprise.entity.TabProject;
import com.gic.enterprise.entity.TabProjectItem;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.ProjectItemQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.AuditLogService;
import com.gic.enterprise.service.ProjectItemApiService;
import com.gic.enterprise.service.ProjectItemService;
import com.gic.enterprise.service.ProjectService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-09-17 11:05
 */
@Service("projectItemApiService")
public class ProjectItemApiServiceImpl implements ProjectItemApiService {

    @Autowired
    private ProjectItemService projectItemService;
    @Autowired
    private ProjectService projectService;
    @Autowired
    private AuditLogService auditLogService;
    @Autowired
    private AuditorApiService auditorApiService;

    @Override
    public ServiceResponse<Integer> saveOrUpdateProjectItem(ProjectItemDTO projectItemDTO) {
        boolean repeat = projectItemService.validProjectItemNameIsRepeat(projectItemDTO.getProjectItemName(), projectItemDTO.getProjectId(), projectItemDTO.getProjectItemId());
        if (repeat) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "审批项名称不能重复");
        }
        boolean repeatCode = projectItemService.validProjectItemCodeIsRepeat(projectItemDTO.getProjectItemCode(), projectItemDTO.getProjectItemId());
        if (repeatCode) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "审批项code不能重复");
        }
        TabProject project = projectService.getById(projectItemDTO.getProjectId());
        if (project == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (projectItemDTO.getProjectItemId() == null) {
            // save
            Integer projectItemId = projectItemService.save(projectItemDTO);
            return ServiceResponse.success(projectItemId);
        } else {
            // update
            Integer line = projectItemService.update(projectItemDTO);
            return ServiceResponse.success(line);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public ServiceResponse<Integer> delProjectItem(Integer projectItemId) {
        boolean flag = auditLogService.hasProjectItemInUse(projectItemId);
        if (flag) {
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "在审批中的审批项不能删除");
        }
        Integer line = projectItemService.del(projectItemId);
        // 删除和审核员的关联
        auditorApiService.delRelByProjectItemId(projectItemId);
        return ServiceResponse.success(line);
    }

    @Override
    public ServiceResponse<Page<ProjectItemDTO>> listProjectItem(ProjectItemQO projectItemQO) {
        com.github.pagehelper.Page page = projectItemService.listProjectItem(projectItemQO);
        Page<ProjectItemDTO> dtoPage = PageHelperUtils.changePageHelperToCurrentPage(page, ProjectItemDTO.class);
        return ServiceResponse.success(dtoPage);
    }

    @Override
    public ServiceResponse<ProjectItemDTO> getById(Integer projectItemId) {
        TabProjectItem projectItem = projectItemService.getById(projectItemId);
        if (projectItem != null) {
            return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(ProjectItemDTO.class, projectItem));
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<ProjectItemDTO> getByProjectItemCode(String projectItemCode) {
        TabProjectItem projectItem = this.projectItemService.getByProjectItemCode(projectItemCode);
        if (projectItem != null) {
            return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(ProjectItemDTO.class, projectItem));
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }
}
