package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.AccountStandardEnum;
import com.gic.enterprise.constant.SmsAppEnum;
import com.gic.enterprise.dto.InternationalSmsRecordDTO;
import com.gic.enterprise.dto.SmsRecordDTO;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.enterprise.utils.SmsUtils;
import com.gic.member.api.dto.MemberUserDTO;
import com.gic.member.api.dto.UserOwnerDTO;
import com.gic.member.api.service.MemberUserApiService;
import com.gic.member.api.service.UserOwnerApiService;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.dto.StoreDTO;
import com.gic.store.service.StoreApiService;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.redisson.api.RAtomicLong;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-06 10:38
 */
@Service("smsRecordApiService")
public class SmsRecordApiServiceImpl implements SmsRecordApiService {

    private Logger logger = LogManager.getLogger(SmsRecordApiServiceImpl.class);

    @Autowired
    private BillingSmsRecordService billingSmsRecordService;
    @Autowired
    private AccountStandardService accountStandardService;
    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingEnterpriseSmsPackageService billingEnterpriseSmsPackageService;
    @Autowired
    private MemberUserApiService memberUserApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private StoreApiService storeApiService;
    @Autowired
    private UserOwnerApiService userOwnerApiService;
    @Autowired
    private InternationalSmsRecordApiService internationalSmsRecordApiService;

    @Override
    public ServiceResponse<Void> verifySendSms(Integer enterpriseId, Integer sendCount, String nationCode) {
        if (enterpriseId == null || sendCount == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }
        if (StringUtils.isBlank(nationCode)) {
            nationCode = "+86";
        } else if (nationCode.charAt(0) != '+') {
            nationCode = "+" + nationCode;
        }
        long missCount = sendCount;
//        if (StringUtils.equals("+86", nationCode)) {
//            logger.info("nationCode：{}, 需要校验套餐包", nationCode);
//            RAtomicLong smsCountR = billingEnterpriseSmsPackageService.getTotalSmsInPackageByEnterpriseId(enterpriseId);
//            long smsPackageCount = smsCountR.get();
//            if (smsPackageCount > sendCount) {
//                logger.info("enterpriseId:{}, 短信套餐包条数:{}, 发送条数：{}, 可以发送", enterpriseId, smsPackageCount, sendCount);
//                return ServiceResponse.success();
//            } else {
//                missCount = missCount - smsPackageCount;
//            }
//        }

        TabBillingAccount billingAccount = billingAccountService.getByEnterpriseId(enterpriseId);
        Double accountBalance = billingAccount.getAccountBalance();
        Integer unitFee = accountStandardService.queryUnitFee(enterpriseId, AccountStandardEnum.SMS.getCode());

        double needFee = (double) (missCount * unitFee) / 1000;
        if (needFee - accountBalance > 5000d) {
            logger.info("enterpriseId:{}, 余额：{}, 发送条数：{}, 不能发送", enterpriseId, accountBalance, sendCount);
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "余额不足， 不能发送");
        } else {
            logger.info("enterpriseId:{},余额：{}, 发送条数：{}, 可以发送", enterpriseId, accountBalance, sendCount);
            return EnterpriseServiceResponse.success();
        }
    }

    @Override
    public ServiceResponse<Void> saveSmsRecord(SmsRecordDTO smsRecordDTO) {
        if (StringUtils.isBlank(smsRecordDTO.getNationCode())) {
            smsRecordDTO.setNationCode("+86");
        }
        if (StringUtils.isBlank(smsRecordDTO.getAppCode())) {
            // 没有应用场景的 通过通知code去匹配
            smsRecordDTO.setAppCode(SmsUtils.getAppCode(smsRecordDTO.getNoticeCode()));
        }
        smsRecordDTO.setSmsType(SmsUtils.getCodeByNoticeCode(smsRecordDTO.getNoticeCode()));



        if (StringUtils.equals("86", smsRecordDTO.getNationCode()) || StringUtils.equals("+86", smsRecordDTO.getNationCode())) {
            ServiceResponse paramResult = ValidUtil.allCheckValidate(smsRecordDTO, SmsRecordDTO.SmsRecordValid.class);
            if (!paramResult.isSuccess()) {
                return paramResult;
            }
            GicMQClient client = GICMQClientUtil.getClientInstance();
            try {
                String json = JSON.toJSONString(smsRecordDTO);
                logger.info("收到国内短信计费信息：{}", json);
                client.sendMessage("smsMq", json);
//            this.run(json);
            } catch (Exception e) {
                logger.warn("调用mq失败", e);
            }
        } else {
            internationalSmsRecordApiService.saveInternationalSmsRecord(EntityUtil.changeEntityByJSON(InternationalSmsRecordDTO.class, smsRecordDTO));
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("国内短信缓存扣费：{}", arg);

            SmsRecordDTO smsRecordDTO = JSON.parseObject(arg, SmsRecordDTO.class);
            // 判断是扣套餐包还是扣余额
            RAtomicLong smsCount = billingEnterpriseSmsPackageService.getTotalSmsInPackageByEnterpriseId(smsRecordDTO.getEnterpriseId());

            smsRecordDTO.setCacheStatus(0);
            smsRecordDTO.setDatabaseStatus(0);

            // 缓存扣费 0套餐包  4余额
            Integer fee = accountStandardService.queryUnitFee(smsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            int deductType = deduct(smsCount, smsRecordDTO.getEnterpriseId(), fee);
            smsRecordDTO.setPayType(deductType);
            smsRecordDTO.setFee((double)fee/1000);

            Long smsRecordId = billingSmsRecordService.saveSmsRecord(smsRecordDTO);
            smsRecordDTO.setSmsRecordId(smsRecordId);

            billingSmsRecordService.updateCacheStatus(smsRecordDTO.getSmsRecordId());
            smsRecordDTO.setCacheStatus(1);
            // 队列
            client.sendMessage("smsDbMq", JSON.toJSONString(smsRecordDTO));
//            this.executeDatabase(JSON.toJSONString(smsRecordDTO));
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    private int deduct(RAtomicLong rSmsCount, Integer enterpriseId, Integer fee) {
        long smsCount = rSmsCount.get();
        if (smsCount > 0) {
            // 套餐包计费
            boolean smsCountDeduct = rSmsCount.compareAndSet(smsCount, smsCount - 1);
            if (!smsCountDeduct) {
                // 设置失败 重新计算
                // todo 这里不知道要不要加个延时之类的
                return deduct(rSmsCount, enterpriseId, fee);
            } else {
                return 0;
            }
        } else {
            // 账户计费
            billingAccountService.deductAccountByCache(fee.doubleValue() / 1000, enterpriseId);
            return 4;
        }
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("国内短信数据库扣费：{}", arg);
        SmsRecordDTO smsRecordDTO = JSON.parseObject(arg, SmsRecordDTO.class);
        // 扣费时间
        smsRecordDTO.setDeductionTime(new Date());
        // 应用名字
        smsRecordDTO.setAppName(SmsAppEnum.getNameByCode(smsRecordDTO.getAppCode()));
        // 会员名字
        setMemberName(smsRecordDTO);
        // 操作人
        setUserName(smsRecordDTO);
        // 门店
        setStore(smsRecordDTO);

        Integer deductType = smsRecordDTO.getPayType();
        if (deductType == 0) {
            // 短信套餐包扣费
            Integer enterpriseSmsPackageId = billingEnterpriseSmsPackageService.deduct(smsRecordDTO.getEnterpriseId());
            smsRecordDTO.setEnterpriseSmsPackageId(enterpriseSmsPackageId);
        } else if (deductType == 4) {
            // 余额扣费
            billingAccountService.deductAccountByDatabase(smsRecordDTO.getFee(), smsRecordDTO.getEnterpriseId());
        }
        billingSmsRecordService.updateDatabaseStatus(smsRecordDTO);
        return ServiceResponse.success();
    }

    private void setStore(SmsRecordDTO smsRecordDTO) {
        ServiceResponse<UserOwnerDTO> userOwnerResp = userOwnerApiService.getUserMaxOwner(smsRecordDTO.getEnterpriseId(), smsRecordDTO.getEcuId(), 1, smsRecordDTO.getMemberCardId(), null);
        if (userOwnerResp.isSuccess()) {
            UserOwnerDTO userOwnerDTO = userOwnerResp.getResult();
            if (userOwnerDTO != null) {
                ServiceResponse<StoreDTO> storeResp = storeApiService.getStoreById(smsRecordDTO.getEnterpriseId(), Integer.valueOf(userOwnerDTO.getOwnerId()));
                if (storeResp.isSuccess()) {
                    StoreDTO storeDTO = storeResp.getResult();
                    if (storeDTO != null) {
                        smsRecordDTO.setStoreId(storeDTO.getStoreId());
                        smsRecordDTO.setStoreName(storeDTO.getStoreName());
                        smsRecordDTO.setStoreGroupId(storeDTO.getStoreGroupId());
                        smsRecordDTO.setStoreGroupName(storeDTO.getStoreGroupName());
                        smsRecordDTO.setStoreBrandName(storeDTO.getStoreBrandNames());
                    }
                } else {
                    logger.info("storeApiService.getStoreById 调用失败 :{}", storeResp.getMessage());
                }
            }
        } else {
            logger.info("UserOwnerApiService.getUserMaxOwner 调用失败:{}", userOwnerResp.getMessage());
        }
    }

    /**
     * 操作人
     */
    private void setUserName(SmsRecordDTO smsRecordDTO) {
        if (smsRecordDTO.getUserId() != null) {
            ServiceResponse<UserDTO> userResp = userApiService.getUserById(smsRecordDTO.getUserId());
            if (userResp.isSuccess()) {
                UserDTO userDTO = userResp.getResult();
                if (userDTO != null) {
                    smsRecordDTO.setUserName(smsRecordDTO.getUserName());
                }
            } else {
                logger.info("userId:{}, UserApiService.getUserById 调用失败:{}", smsRecordDTO.getUserId(), userResp.getMessage());
            }
        }
    }

    /**
     * 会员名字和会员卡号
     */
    private void setMemberName(SmsRecordDTO smsRecordDTO) {
        if (smsRecordDTO.getMcuId() != null) {
            ServiceResponse<MemberUserDTO> memberUserDTOServiceResponse = memberUserApiService.getByMcuId(smsRecordDTO.getMcuId(), smsRecordDTO.getEnterpriseId());
            if (memberUserDTOServiceResponse.isSuccess()) {
                MemberUserDTO memberUserDTO = memberUserDTOServiceResponse.getResult();
                if (memberUserDTO != null) {
                    smsRecordDTO.setMemberName(memberUserDTO.getName());
                    smsRecordDTO.setMemberCardName(memberUserDTO.getCardName());
                }
            } else {
                logger.warn("mcuId:{}, enterpriseId:{}, memberUserApiService.getByMcuId调用失败:{}, ", smsRecordDTO.getMcuId(), smsRecordDTO.getEnterpriseId(), memberUserDTOServiceResponse.getMessage());
            }
        }
    }
}