package com.gic.enterprise.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.dto.TodoItemDTO;
import com.gic.enterprise.entity.TabTodoItem;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.TodoItemQO;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.TodoItemApiService;
import com.gic.enterprise.service.TodoItemService;
import com.gic.enterprise.utils.valid.ValidParamsUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-14 17:20
 */
@Service("todoItemApiService")
public class TodoItemApiServiceImpl implements TodoItemApiService {

    @Autowired
    private TodoItemService todoItemService;

    @Override
    public ServiceResponse<Void> saveTodoItem(TodoItemDTO todoItemDTO) {
        ServiceResponse serviceResponse = ValidParamsUtils.allCheckValidate(todoItemDTO, TodoItemDTO.SaveValidView.class);
        if (!serviceResponse.isSuccess()) {
            return serviceResponse;
        }
        TabTodoItem tabTodoItem = todoItemService.getByItemInfoUrl(todoItemDTO.getEnterpriseId(), todoItemDTO.getUserId(), todoItemDTO.getItemType(), todoItemDTO.getItemInfoUrl());
        if (tabTodoItem != null) {
            return EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS);
        }
        todoItemService.save(todoItemDTO);
        return EnterpriseServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> finishTodoItem(Integer enterpriseId, Integer userId, String itemType, String itemInfoUrl) {
        int line = todoItemService.finishTodoItem(enterpriseId, userId, itemType, itemInfoUrl);
        if (line > 0) {
            return EnterpriseServiceResponse.success();
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.DATA_EXISTS);
        }
    }

    @Override
    public ServiceResponse<Page<TodoItemDTO>> listTodoItem(TodoItemQO todoItemQO) {
        com.github.pagehelper.Page<TabTodoItem> page = this.todoItemService.listTodoItem(todoItemQO);
        Page<TodoItemDTO> result = PageHelperUtils.changePageHelperToCurrentPage(page, TodoItemDTO.class);
        return EnterpriseServiceResponse.success(result);
    }

    @Override
    public ServiceResponse<TodoItemDTO> getTodoItem(Integer todoItemId) {
        TabTodoItem tabTodoItem = this.todoItemService.getById(todoItemId);
        if (tabTodoItem != null) {
            return EnterpriseServiceResponse.success(EntityUtil.changeEntityByJSON(TodoItemDTO.class, tabTodoItem));
        } else {
            return EnterpriseServiceResponse.failure(ErrorCode.NOTEXISTS);
        }
    }

    @Override
    public ServiceResponse<Integer> getUnFinishCount(TodoItemQO todoItemQO) {
        Integer count = this.todoItemService.getUnFinishCount(todoItemQO);
        return EnterpriseServiceResponse.success(count);
    }
}
