package com.gic.enterprise.service.outer.impl;

import java.util.List;
import java.util.Optional;

import com.gic.enterprise.dto.WmMemberCardConfigDTO;
import com.gic.enterprise.dto.WmOrderConfigDTO;
import com.gic.enterprise.entity.TabWmMemberCardConfig;
import com.gic.enterprise.entity.TabWmOrderConfig;
import com.gic.enterprise.service.WmMemberCardConfigService;
import com.gic.enterprise.service.WmOrderConfigService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.constant.MallModeEnum;
import com.gic.enterprise.dto.WmStoreDTO;
import com.gic.enterprise.entity.TabWmStore;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.service.WmStoreApiService;
import com.gic.enterprise.service.WmStoreService;

@Service("wmStoreApiService")
public class WmStoreApiServiceImpl implements WmStoreApiService {
    @Autowired
    private WmStoreService wmStoreService;
    @Autowired
    private WmMemberCardConfigService wmMemberCardConfigService;
    @Autowired
    private WmOrderConfigService wmOrderConfigService;

    @Override
    public ServiceResponse<Integer> saveWmStore(WmStoreDTO wmStoreDTO) {
        ServiceResponse validParam = validParam(wmStoreDTO);
        if (!validParam.isSuccess()) {
            return ServiceResponse.failure(validParam.getCode(), validParam.getMessage());
        }
        return ServiceResponse.success(wmStoreService.saveWmStore(wmStoreDTO));
    }

    @Override
    public ServiceResponse<Void> modifyWmStore(WmStoreDTO wmStoreDTO) {
        ServiceResponse validParam = validParam(wmStoreDTO);
        if (!validParam.isSuccess()) {
            return ServiceResponse.failure(validParam.getCode(), validParam.getMessage());
        }
        wmStoreService.modifyWmStore(wmStoreDTO);
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<String> authWmStore(Integer wmMallStoreId) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
        }
        wmStoreService.modifyWmStore(new WmStoreDTO().setWmMallStoreId(wmMallStoreId).setAuthStatus(1));
        return ServiceResponse.success(record.getWmPidName());
    }

    @Override
    public ServiceResponse<List<WmStoreDTO>> listWmStore(Integer enterpriseId) {
        List<TabWmStore> list = wmStoreService.listWmStore(enterpriseId);
        return ServiceResponse.success(
                EntityUtil.changeEntityListNew(WmStoreDTO.class, list));
    }

    @Override
    public ServiceResponse<WmStoreDTO> getWmStoreByWmMallStoreId(Integer wmMallStoreId) {
        TabWmStore record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(WmStoreDTO.class, record));
    }

    @Override
    public ServiceResponse<Integer> saveOrUpdateWmMemberCardConfig(WmMemberCardConfigDTO dto) {
        ServiceResponse validParam = validWmMemberCardConfig(dto);
        if (!validParam.isSuccess()) {
            return ServiceResponse.failure(validParam.getCode(), validParam.getMessage());
        }
        TabWmMemberCardConfig record = wmMemberCardConfigService.getByWmMainAccount(dto.getWmMainAccount());
        if (record == null) {
            return ServiceResponse.success(wmMemberCardConfigService.saveWmMemberCardConfig(dto));
        }
        //编辑
        dto.setMemberCardConfigId(record.getMemberCardConfigId());
        wmMemberCardConfigService.updateWmMemberCardConfig(dto);
        return ServiceResponse.success(record.getMemberCardConfigId());
    }

    @Override
    public ServiceResponse<WmMemberCardConfigDTO> getWmMemberCardConfigByWmMainAccount(String wmMainAccount) {
        TabWmMemberCardConfig record = wmMemberCardConfigService.getByWmMainAccount(wmMainAccount);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "还未配置");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(WmMemberCardConfigDTO.class, record));
    }

    @Override
    public ServiceResponse<Integer> saveOrUpdateWmOrderConfig(WmOrderConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        dto.setOrderFromMode(Optional.ofNullable(dto.getOrderFromMode()).orElse(1));
        dto.setWmOrderToOffline(Optional.ofNullable(dto.getWmOrderToOffline()).orElse(0));
        dto.setOfflineDifferentiateWm(Optional.ofNullable(dto.getOfflineDifferentiateWm()).orElse(0));
        TabWmStore wmStore = wmStoreService.getByWmMainAccount(dto.getWmMainAccount());
        if (wmStore == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号错误，查无数据");
        }
        TabWmOrderConfig record = wmOrderConfigService.getByWmMainAccount(dto.getWmMainAccount());
        if (record == null) {
            return ServiceResponse.success(wmOrderConfigService.saveWmOrderConfig(dto));
        }
        dto.setOrderConfigId(record.getOrderConfigId());
        wmOrderConfigService.editWmOrderConfig(dto);
        return ServiceResponse.success(record.getOrderConfigId());
    }

    @Override
    public ServiceResponse<WmOrderConfigDTO> getWmOrderConfigByWmMainAccount(String wmMainAccount) {
        TabWmOrderConfig record = wmOrderConfigService.getByWmMainAccount(wmMainAccount);
        if (record == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "还未配置");
        }
        return ServiceResponse.success(EntityUtil.changeEntityNew(WmOrderConfigDTO.class, record));
    }

    private ServiceResponse validWmMemberCardConfig(WmMemberCardConfigDTO dto) {
        if (dto.getEnterpriseId() == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商户ID不能为空");
        }
        if (StringUtils.isBlank(dto.getWmMainAccount())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号不能为空");
        }
        TabWmStore wmStore = wmStoreService.getByWmMainAccount(dto.getWmMainAccount());
        if (wmStore == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号错误，查无数据");
        }
        if (dto.getOpenCardMode() == null) {
            //默认达摩开卡
            dto.setOpenCardMode(1);
        }
        //GIC会员归属信息同步微盟 默认开启
        dto.setGicMemberToWm(Optional.ofNullable(dto.getGicMemberToWm()).orElse(1));
        //微盟会员导购关系同步GIC 默认开启
        dto.setWmMemberClerkToGic(Optional.ofNullable(dto.getWmMemberClerkToGic()).orElse(1));
        //微盟会员信息（包括门店关系）同步GIC 默认开启
        dto.setWmMemberToGic(Optional.ofNullable(dto.getWmMemberToGic()).orElse(1));
        return ServiceResponse.success();
    }

    private ServiceResponse<Void> validParam(WmStoreDTO wmStoreDTO) {
        Integer wmMallStoreId = wmStoreDTO.getWmMallStoreId();
        boolean isEdit = wmMallStoreId != null;
        TabWmStore record = null;
        if (wmMallStoreId != null) {
            record = wmStoreService.getByWmMallStoreId(wmMallStoreId);
            if (record == null) {
                return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟商城店铺ID错误，查无数据");
            }
            wmStoreDTO.setEnterpriseId(record.getEnterpriseId());
        }
        //已授权
        boolean isCanEdit = isEdit && record.getAuthStatus().intValue() == 1;

        if (StringUtils.isBlank(wmStoreDTO.getWmMainAccount())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号不能为空");
        }
        if (wmStoreService.validRepeat(wmStoreDTO.getWmMainAccount(), wmMallStoreId, 1)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "微盟主账号已存在");
        }
        if (isCanEdit && !record.getWmMainAccount().equals(wmStoreDTO.getWmMainAccount())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，微盟主账号不能修改");
        }
        if (StringUtils.isBlank(wmStoreDTO.getWmPid())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "PID不能为空");
        }
        if (isCanEdit && !record.getWmPid().equals(wmStoreDTO.getWmPid())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，PID不能修改");
        }
        if (wmStoreService.validRepeat(wmStoreDTO.getWmPid(), wmMallStoreId, 3)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "PID已存在");
        }
        if (StringUtils.isBlank(wmStoreDTO.getWmStoreId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "storeID不能为空");
        }
        if (isCanEdit && !record.getWmStoreId().equals(wmStoreDTO.getWmStoreId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，storeID不能修改");
        }
        if (wmStoreService.validRepeat(wmStoreDTO.getWmStoreId(), wmMallStoreId, 2)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "storeID已存在");
        }
        if (StringUtils.isBlank(wmStoreDTO.getWmPidName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺名称不能为空");
        }
        if (isCanEdit && !record.getWmPidName().equals(wmStoreDTO.getWmPidName())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，店铺名称不能修改");
        }
        if (wmStoreService.validRepeatStoreName(wmStoreDTO.getWmPidName(), wmMallStoreId,
                wmStoreDTO.getEnterpriseId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店铺名称已存在");
        }
        if (!MallModeEnum.isContain(wmStoreDTO.getMallMode())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "商城模式值非法");
        }
        //选择智慧零售模式保存后或者有进行过门店同步则不允许切换回微商城模式
        if (isEdit && (record.getMallMode().intValue() == MallModeEnum.SMART_RETAIL.getCode())
                && wmStoreDTO.getMallMode().intValue() != MallModeEnum.SMART_RETAIL.getCode()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已进行过智慧零售模式保存，不允许切换回微商城模式");
        }
        Integer storeBrandId = wmStoreDTO.getStoreBrandId();
        if (storeBrandId == null) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "店招品牌不能为空");
        }
        if (isCanEdit && record.getStoreBrandId().intValue() != storeBrandId.intValue()) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "已授权，店招品牌不能修改");
        }
        return ServiceResponse.success();
    }
}
