package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.AuditorDTO;
import com.gic.auth.service.AuditorApiService;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.download.utils.log.LogUtils;
import com.gic.enterprise.ano.IgnoreLogin;
import com.gic.enterprise.dto.AuditLogDTO;
import com.gic.enterprise.dto.ProjectItemDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.AuditLogQO;
import com.gic.enterprise.service.AuditLogApiService;
import com.gic.enterprise.service.ProjectItemApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-10-30 1:21
 */
@RestController
public class AuditLogController {

    @Autowired
    private AuditLogApiService auditLogApiService;
    @Autowired
    private AuditorApiService auditorApiService;
    @Autowired
    private ProjectItemApiService projectItemApiService;

    @RequestMapping("/audit")
    @IgnoreLogin
    public RestResponse audit(@Validated(AuditLogQO.AuditView.class) AuditLogQO auditLogQO) {
        AuditLogDTO auditLogDTO = new AuditLogDTO();
        auditLogDTO.setAuditLogId(auditLogQO.getAuditLogId());
        auditLogDTO.setAuditResult(auditLogQO.getAuditResult());
        auditLogDTO.setAuditReason(auditLogQO.getAuditReason());
        ServiceResponse<AuditorDTO> auditorServiceResponse;
        if (StringUtils.isNotBlank(auditLogQO.getOpenid())) {
            auditorServiceResponse = auditorApiService.getAuditorByOpenid(auditLogQO.getOpenid());
        } else if (UserDetailUtils.getUserDetail() != null) {
            auditorServiceResponse = auditorApiService.getAuditorByUserId(UserDetailUtils.getUserDetail().getEnterpriseId(), UserDetailUtils.getUserDetail().getUserId());
        }else {
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数错误");
        }
        if (!auditorServiceResponse.isSuccess()) {
            return ResultControllerUtils.commonResult(auditorServiceResponse);
        }
        auditLogDTO.setAuditorId(auditorServiceResponse.getResult().getAuditorId());
        auditLogDTO.setAuditorName(auditorServiceResponse.getResult().getAuditorName());
        auditLogDTO.setAuditorPhone(auditorServiceResponse.getResult().getPhone());


//        ServiceResponse<AuditLogDTO> auditLog = this.auditLogApiService.getAuditLog(auditLogQO.getAuditLogId());
        ServiceResponse<AuditLogDTO> serviceResponse = auditLogApiService.audit(auditLogDTO);
        if (serviceResponse.isSuccess()) {
            createLog(auditLogDTO);
        }
        if (serviceResponse.isSuccess()) {
            return RestResponse.success();
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage(), serviceResponse.getResult());
        }
    }

    private void createLog(AuditLogDTO auditLogDTO) {
        ServiceResponse<AuditLogDTO> auditLog = this.auditLogApiService.getAuditLog(auditLogDTO.getAuditLogId());
        LogUtils.createLog("审核审批项", auditLog.getResult().getProjectItemName(), auditLogDTO.getAuditorName(), auditLog.getResult().getEnterpriseId(), auditLog.getResult().getEnterpriseName());
    }

    @RequestMapping("/get-audit-log")
    @IgnoreLogin
    public RestResponse getAuditLog(Integer auditLogId) {
        ServiceResponse<AuditLogDTO> serviceResponse = this.auditLogApiService.getAuditLog(auditLogId);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-audit-log")
    public RestResponse listAuditLog(AuditLogQO auditLogQO) {
        auditLogQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<AuditLogDTO>> serviceResponse = this.auditLogApiService.listAuditLog(auditLogQO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/project-item-info")
    public RestResponse projectItemInfo(String projectItemCode) {
        ServiceResponse<ProjectItemDTO> serviceResponse = this.projectItemApiService.getByProjectItemCode(projectItemCode);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

}
