package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.service.MenuApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.constant.SmsAppEnum;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.RecordQO;
import com.gic.enterprise.service.BillingRecordApiService;
import com.gic.enterprise.service.DictApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.*;
import com.gic.open.api.dto.ApplicationDTO;
import com.gic.open.api.service.ApplicationApiService;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019/8/22 16:33
 */
@RestController
public class BillingRecordController {

    private Logger logger = LogManager.getLogger(BillingRecordController.class);

    @Autowired
    private BillingRecordApiService billingRecordApiService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    ApplicationApiService applicationApiService;
    @Autowired
    MenuApiService menuApiService;

    LoadingCache<String, Long> appCache = CacheBuilder.newBuilder()
            .maximumSize(20)
            .expireAfterWrite(1L, TimeUnit.HOURS)
            .build(new CacheLoader<String, Long>() {
                @Override
                public Long load(String s) throws Exception {
                    ApplicationDTO applicationDTO = applicationApiService.getApplicationByModule(s).getResult();
                    if (applicationDTO != null) {
                        return applicationDTO.getApplicationId();
                    } else {
                        return null;
                    }
                }
            });


    @RequestMapping("/list-account-balance")
    public RestResponse listAccountBalanceRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<AccountBalanceRecordDTO>> serviceResponse = billingRecordApiService.listAccountBalanceRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(), serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<AccountBalanceRecordVO> list = EntityUtil.changeEntityListByJSON(AccountBalanceRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-double-calling")
    public RestResponse listDoubleCallingRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<DoubleCallingRecordDTO>> serviceResponse = billingRecordApiService.listDoubleCallingRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(), serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<DoubleCallingRecordVO> list = EntityUtil.changeEntityListByJSON(DoubleCallingRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-recording-storage")
    public RestResponse listRecordingStorageRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<RecordingStorageRecordDTO>> serviceResponse = billingRecordApiService.listRecordingStorageRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(), serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<RecordingStorageRecordVO> list = EntityUtil.changeEntityListByJSON(RecordingStorageRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-sms-package")
    public RestResponse listSmsPackageRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<SmsPackageRecordDTO>> serviceResponse = billingRecordApiService.listSmsPackageRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(), serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<SmsPackageRecordVO> list = EntityUtil.changeEntityListByJSON(SmsPackageRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-sms")
    public RestResponse listSmsRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (StringUtils.isNotBlank(recordQO.getStartTime())) {
            recordQO.setStartTime(recordQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(recordQO.getEndTime())) {
            recordQO.setEndTime(recordQO.getEndTime() + " 23:59:59");
        }
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() == 0 && StringUtils.isBlank(recordQO.getAppCode())) {
            Set<String> appIdList = menuApiService.getAppIdSetByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
            if (CollectionUtils.isEmpty(appIdList)) {
                return RestResponse.success(new Page<>());
            }
            List<ApplicationDTO> applicationList = applicationApiService.listApplicationByIdIn(appIdList.stream().map(Long::valueOf).collect(Collectors.toList())).getResult();
            if (CollectionUtils.isEmpty(applicationList)) {
                return RestResponse.success(new Page<>());
            }
            List<String> appCodeList = applicationList.stream().map(ApplicationDTO::getModuleCode).collect(Collectors.toList());
            recordQO.setAppCodeList(appCodeList);
        }

        ServiceResponse<Page<SmsRecordDTO>> serviceResponse = billingRecordApiService.listSmsRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(),
                    serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<SmsRecordVO> list = EntityUtil.changeEntityListByJSON(SmsRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-international-sms")
    public RestResponse listInternationalSmsRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        if (StringUtils.isNotBlank(recordQO.getStartTime())) {
            recordQO.setStartTime(recordQO.getStartTime() + " 00:00:00");
        }
        if (StringUtils.isNotBlank(recordQO.getEndTime())) {
            recordQO.setEndTime(recordQO.getEndTime() + " 23:59:59");
        }
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() == 0 && StringUtils.isBlank(recordQO.getAppCode())) {
            Set<String> appIdList = menuApiService.getAppIdSetByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
            if (CollectionUtils.isEmpty(appIdList)) {
                return RestResponse.success(new Page<>());
            }
            List<ApplicationDTO> applicationList = applicationApiService.listApplicationByIdIn(appIdList.stream().map(Long::valueOf).collect(Collectors.toList())).getResult();
            if (CollectionUtils.isEmpty(applicationList)) {
                return RestResponse.success(new Page<>());
            }
            List<String> appCodeList = applicationList.stream().map(ApplicationDTO::getModuleCode).collect(Collectors.toList());
            recordQO.setAppCodeList(appCodeList);
        }
        ServiceResponse<Page<InternationalSmsRecordDTO>> serviceResponse = billingRecordApiService.listInternationalSmsRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(), serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<InternationalSmsRecordVO> list = EntityUtil.changeEntityListByJSON(InternationalSmsRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-voice-code")
    public RestResponse listVoiceCodeRecord(RecordQO recordQO) {
        recordQO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        ServiceResponse<Page<VoiceCodeRecordDTO>> serviceResponse = billingRecordApiService.listVoiceCodeRecord(recordQO);
        if (serviceResponse.isSuccess()) {
            Page page = new Page(serviceResponse.getResult().getCurrentPage(), serviceResponse.getResult().getPageSize(), serviceResponse.getResult().getTotalPage(), serviceResponse.getResult().getTotalCount());
            List<VoiceCodeRecordVO> list = EntityUtil.changeEntityListByJSON(VoiceCodeRecordVO.class, serviceResponse.getResult().getResult());
            page.setResult(list);
            return RestResponse.success(page);
        } else {
            return RestResponse.failure(serviceResponse.getCode(), serviceResponse.getMessage());
        }
    }

    @RequestMapping("/list-fee")
    public RestResponse listFee(String startTime, String endTime) {
        ServiceResponse<List<Map<String, Object>>> serviceResponse = billingRecordApiService.listFee(UserDetailUtils.getUserDetail().getEnterpriseId(), startTime, endTime);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("/list-sms-app-type")
    public RestResponse listSmsAppType() {
        Set<String> appIdSet;
        if (UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() == 0) {
            appIdSet = menuApiService.getAppIdSetByUserId(UserDetailUtils.getUserDetail().getUserId()).getResult();
        } else {
            appIdSet = new HashSet<>();
        }
        try {
            List<Map<String, String>> list = new ArrayList<>();
            for (SmsAppEnum app : SmsAppEnum.values()) {
                HashMap<String, String> map = new HashMap<>();
                if (app.isCtrl() && UserDetailUtils.getUserDetail().getUserInfo().getSuperAdmin() == 0) {
                    Long appId = appCache.get(app.getCode());
                    if (!appIdSet.contains(appId.toString())) {
                        continue;
                    }
                }
                map.put("key", app.getName());
                map.put("value", app.getCode());
                list.add(map);
            }
            return RestResponse.success(list);
        } catch (ExecutionException e) {
            logger.info("获取缓存失败", e);
        }
        return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), ErrorCode.SYSTEM_ERROR.getMsg());
    }

    

}
