package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.CustomStoreDTO;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.CustomStoreApiService;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.qo.CustomStoreQO;
import com.gic.enterprise.web.vo.CustomStoreVO;
import com.gic.store.dto.StoreWidgetDTO;
import com.gic.store.service.StoreWidgetApiService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class CustomStoreController {
    @Autowired
    private CustomStoreApiService customStoreApiService;

    @RequestMapping("save-custom-store")
    public RestResponse saveCustomStore(@RequestBody CustomStoreQO customStoreQO){
        CustomStoreDTO dto = new CustomStoreDTO();
        dto.setTitle(customStoreQO.getTitle());
        dto.setAppid(customStoreQO.getAppId());
        dto.setAppType(customStoreQO.getAppType());
        dto.setCustomStoreId(customStoreQO.getCustomStoreId());
        dto.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        dto.setStoreWidgetId(customStoreQO.getStorewidgetId());
        if(customStoreQO.getSaveType() == 2){
            dto.setStatus(1);
        }else {
            if(customStoreQO.getCustomStoreId() == null){
                dto.setStatus(2);
            }
        }
        ServiceResponse<Integer> serviceResponse = this.customStoreApiService.saveCustomStore(dto);
        if(serviceResponse.isSuccess()){
            return RestResponse.success();
        }
        return EnterpriseRestResponse.failure(serviceResponse);
    }

    @RequestMapping("page-custom-store")
    public RestResponse pageCustomStore(String search, String appId, PageQO pageQO){
        if(StringUtils.isBlank(appId)){
            return RestResponse.success();
        }
        ServiceResponse<Page<CustomStoreDTO>> response = this.customStoreApiService.pageCustomStore(UserDetailUtils.getUserDetail().getEnterpriseId(),
                search, appId, pageQO.getCurrentPage(), pageQO.getPageSize());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("custom-store-detail")
    public RestResponse customStoreDetail(Integer customStoreId){
        ServiceResponse<CustomStoreDTO> response = this.customStoreApiService.getCustomStore(customStoreId);
        if(response.isSuccess()){
            CustomStoreDTO dto = response.getResult();
            if(dto != null){
                CustomStoreVO vo = EntityUtil.changeEntityByJSON(CustomStoreVO.class, dto);
                return RestResponse.success(vo);
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("delete-custom-store")
    public RestResponse deleteCustomStore(String customStoreId){
        if(StringUtils.isBlank(customStoreId)){
            return RestResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "参数不能为空");
        }
        if(StringUtils.isNotBlank(customStoreId)){
            String[] arr = customStoreId.split(",");
            for(String s : arr){
                this.customStoreApiService.deleteCustomStore(Integer.valueOf(s));
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("open-custom-store")
    public RestResponse updateCustomStoreStatus(Integer customStoreId, String appId){
        this.customStoreApiService.openCustomStore(UserDetailUtils.getUserDetail().getEnterpriseId(), appId, customStoreId);
        return RestResponse.success();
    }
}
