package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.auth.dto.UserDTO;
import com.gic.auth.service.UserApiService;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.BillingAccountDTO;
import com.gic.enterprise.dto.EnterpriseDTO;
import com.gic.enterprise.response.EnterpriseRestResponse;
import com.gic.enterprise.service.BillingAccountApiService;
import com.gic.enterprise.service.EnterpriseApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetail;
import com.gic.enterprise.utils.UserDetailUtils;
import com.gic.enterprise.web.vo.EnterpriseResourceRelVO;
import com.gic.enterprise.web.vo.EnterpriseVO;
import com.gic.enterprise.web.vo.HomePageEnterpriseInfoVO;
import com.gic.marketing.api.dto.sms.EnterpriseSmsSignDTO;
import com.gic.marketing.api.dto.sms.SmsApplicationDTO;
import com.gic.marketing.api.service.sms.SmsOuterApiService;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

@RestController
public class EnterpriseController {
    @Autowired
    private EnterpriseApiService enterpriseApiService;
    @Autowired
    private UserApiService userApiService;
    @Autowired
    private SmsOuterApiService smsOuterApiService;
    @Autowired
    private BillingAccountApiService billingAccountApiService;

    @RequestMapping("get-enterprise-by-name")
    public RestResponse getEnterpriseByName(String enterpriseName) {
        if (StringUtils.isNotBlank(enterpriseName)) {
            ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseByName(enterpriseName);
            if (enterprise.isSuccess()) {
                return RestResponse.success(enterprise.getResult());
            }
        }
        return RestResponse.success();
    }

    @RequestMapping("/enterprise-info")
    public RestResponse enterpriseInfo() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        EnterpriseVO enterpriseVO = new EnterpriseVO();
        ServiceResponse<EnterpriseDTO> enterprise = this.enterpriseApiService.getEnterpriseById(enterpriseId);
        enterpriseVO.setEnterprise(enterprise.getResult());
        ServiceResponse<UserDTO> user = userApiService.getUserByEnterpriseId(enterpriseId);
        enterpriseVO.setUser(user.getResult());
        ServiceResponse<SmsApplicationDTO> enterpriseSmsApp = smsOuterApiService.getEnterpriseSmsApp(enterpriseId);
        enterpriseVO.setSmsApplication(enterpriseSmsApp.getResult());
        ServiceResponse<Page<EnterpriseSmsSignDTO>> serviceResponse = smsOuterApiService.listSmsSignByEid(enterpriseId,
                new Page<>(1, Integer.MAX_VALUE));
        enterpriseVO.setSmsSignList(serviceResponse.getResult().getResult());
        return RestResponse.success(enterpriseVO);
    }

    @RequestMapping("/get-enterprise-info")
    public RestResponse getEnterpriseInfo() {
        Integer enterpriseId = UserDetailUtils.getUserDetail().getEnterpriseId();
        return ResultControllerUtils.commonResult(this.enterpriseApiService.getBusinessFrontRes(enterpriseId),
                EnterpriseResourceRelVO.class);
    }

    @RequestMapping("/update-enterprise")
    public RestResponse updateEnterprise(String enterpriseName, String companyName, String areaId, String address,
            String logo) {
        EnterpriseDTO enterpriseDTO = new EnterpriseDTO();
        enterpriseDTO.setEnterpriseId(UserDetailUtils.getUserDetail().getEnterpriseId());
        enterpriseDTO.setEnterpriseName(enterpriseName);
        enterpriseDTO.setCompanyName(companyName);
        enterpriseDTO.setAreaId(areaId);
        enterpriseDTO.setAddress(address);
        enterpriseDTO.setLogo(logo);
        ServiceResponse<Integer> serviceResponse = enterpriseApiService.editEnterprise(enterpriseDTO);
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("home-page-enterprise-info")
    public RestResponse getHomePageEnterpriseInfo() {
        UserDetail userDetail = UserDetailUtils.getUserDetail();
        ServiceResponse<BillingAccountDTO> result = billingAccountApiService
                .getByEnterpriseId(userDetail.getEnterpriseId());
        if (result.isSuccess()) {
            HomePageEnterpriseInfoVO vo = EntityUtil.changeEntityNew(HomePageEnterpriseInfoVO.class,
                    userDetail.getEnterpriseInfo());
            vo.setAccountBalance(result.getResult().getAccountBalance());
            return RestResponse.success(vo);
        }
        return EnterpriseRestResponse.failure(result);
    }

}
