package com.gic.enterprise.web.controller;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.webapi.reponse.RestResponse;
import com.gic.enterprise.dto.*;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.qo.PageQO;
import com.gic.enterprise.service.HelpCenterApiService;
import com.gic.enterprise.service.IndexModuleApiService;
import com.gic.enterprise.service.PushMessageApiService;
import com.gic.enterprise.utils.ResultControllerUtils;
import com.gic.enterprise.utils.UserDetailUtils;
import org.apache.commons.lang.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-11-14 9:11
 */
@RestController
public class IndexModuleController {
    private static final Logger log = LogManager.getLogger(IndexModuleController.class);
    @Autowired
    private IndexModuleApiService indexModuleApiService;
    @Autowired
    private HelpCenterApiService helpCenterApiService;
    @Autowired
    private PushMessageApiService pushMessageApiService;

    @RequestMapping("/list-index-module")
    public RestResponse listIndexModule() {
        ServiceResponse<List<IndexModuleDTO>> serviceResponse = indexModuleApiService.listIndexModuleEnterprise(UserDetailUtils.getUserDetail().getEnterpriseId());
        return ResultControllerUtils.commonResult(serviceResponse);
    }

    @RequestMapping("list-help-center")
    public RestResponse listHelpCenter(){
        ServiceResponse<List<HelpCenterDTO>> response = this.helpCenterApiService.listHelpCenter();
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("my-message")
    public RestResponse myMessage(String search, Integer status, String startTime, String endTime, Integer pushClassifyId, PageQO pageQO){
        Integer userId = UserDetailUtils.getUserDetail().getUserId();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        try {
            ServiceResponse<Page<UserMessageDTO>> response = this.pushMessageApiService.pageUserMessage(userId, search, status, StringUtils.isBlank(startTime) ? null : sdf.parse(startTime+" 00:00:00"),
                    StringUtils.isBlank(endTime) ? null : sdf.parse(endTime+" 23:59:59"), pushClassifyId, pageQO.getCurrentPage(), pageQO.getPageSize());
            ServiceResponse<Integer> unReadResponse = this.pushMessageApiService.totalUserMessage(UserDetailUtils.getUserDetail().getUserId(), 1);
            Map<String, Object> result = new HashMap<>();
            result.put("page", response.getResult());
            result.put("unReadCount", unReadResponse.getResult());
            return RestResponse.success(result);
        } catch (ParseException e) {
            log.warn("异常", e);
            return RestResponse.failure(ErrorCode.SYSTEM_ERROR.getCode(), "日期转换错误");
        }
    }

    @RequestMapping("total-message")
    public RestResponse totalMessage(){
        ServiceResponse<Integer> response = this.pushMessageApiService.totalUserMessage(UserDetailUtils.getUserDetail().getUserId(), 1);
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("read-all")
    public RestResponse readAll(){
        ServiceResponse<Integer> response = this.pushMessageApiService.readAll(UserDetailUtils.getUserDetail().getUserId());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("message-detail")
    public RestResponse messageDetail(Integer messageId){
        ServiceResponse<PushMessageDTO> response = this.pushMessageApiService.getPushMessageById(messageId);
        this.pushMessageApiService.readMessage(messageId, UserDetailUtils.getUserDetail().getUserId());
        return RestResponse.success(response.getResult());
    }

    @RequestMapping("message-classify")
    public RestResponse listMessageClassify(){
        ServiceResponse<Page<PushTypeDTO>> response = this.pushMessageApiService.pagePushType(null, 1, 10000);
        return RestResponse.success(response.getResult().getResult());
    }
}
